package entrophy;

import java.util.function.Function;

import io.vavr.Tuple2;
import io.vavr.Tuple3;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;

public class EntrophyGI<S> {
	private final double entropyFrom;
	private final double entropyTo;
	private final IndexedSeq<Tuple2<S, Number>> fonemsFrom;
	private final IndexedSeq<Tuple2<S, Number>> fonemsTo;
	private final IndexedSeq<Tuple3<S, S, Number>> fonemTwins;
	private final int wordsTwinsCount;
	
	public EntrophyGI(
			final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> input,
			final Function<Tuple2<IndexedSeq<S>, IndexedSeq<S>>, Tuple2<IndexedSeq<S>, IndexedSeq<S>>>
				removeAuxstrings
			) {
		
		final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> in = prepareData(input, removeAuxstrings);
		
		this.fonemsFrom = calculateFonems(in, false);
		this.fonemsTo = calculateFonems(in, true);
		this.fonemTwins = calculateFonemTwins(in);
		this.entropyFrom = calculateEntropy(fonemTwins, fonemsFrom, true);
		this.entropyTo = calculateEntropy(fonemTwins, fonemsTo, false);
		
		this.wordsTwinsCount = in.length();
			
	}
	/***************************************/
	/**
	 * predpripravy data:
	 *  - odstrani znaky operaci
	 *  - zkontroluje chybove hlasky
	 *  - delka slov
	 * @param in
	 * @return
	 */
	private IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> prepareData(
			final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> in,
			final Function<Tuple2<IndexedSeq<S>, IndexedSeq<S>>, Tuple2<IndexedSeq<S>, IndexedSeq<S>>> remove
			){
		return Stream.range(0, in.size()).foldLeft(Array.empty(),
				(currentRow, i)->currentRow.append(remove.apply(in.get(i))));
					
	}
	

	/***************************/
	public double getEntropyFrom(){
		return entropyFrom;
	}
	public double getEntropyTo(){
		return entropyTo;
	}
	public IndexedSeq<Tuple2<S, Number>> getFonemsFrom(){
		return fonemsFrom;
	}
	public IndexedSeq<Tuple2<S, Number>> getFonemsTo(){
		return fonemsTo;
	}
	public IndexedSeq<Tuple3<S, S, Number>> getFonemTwins(){
		return fonemTwins;
	}
	public int getWordsTwinsCount(){
		return wordsTwinsCount;
	}
	
	/*************************/
	/**
	 * pocita entropii jednoho nebo druheho jazyku
	 * @param fonemTwins 
	 * @param fonems
	 * @param first prvni jazyk TRUE, druhy jazyk FALSE
	 * @return
	 */
	private double calculateEntropy(
				final IndexedSeq<Tuple3<S, S, Number>> fonemTwins,
				final IndexedSeq<Tuple2<S, Number>> fonems, final boolean first) {
		return Stream.range(0, fonemTwins.size()).foldLeft(0.0,
				(entrophy, i)->{
					return entrophy+(
							-/*(double)*/(fonemTwins.get(i)._3.doubleValue()/(double)(fonemTwins.size()))
							*Math.log( /*(double)*/(fonemTwins.get(i)._3.doubleValue())/
									fonems.get( getIndex(
											       fonems, ((first)?fonemTwins.get(i)._1:fonemTwins.get(i)._2 )  )  )
											._2.doubleValue()
									)/Math.log(2.0)
							);
				}  );
	}
	/******************************************************/	
	/**
	 * pocet dvojic fonem v jazyce
	 * @param in
	 * @return
	 */
	private IndexedSeq<Tuple3<S, S, Number>> calculateFonemTwins(
			//final IndexedSeq<Tuple3<IndexedSeq<S>, IndexedSeq<S>, Double>> in
			final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> in) {
		final IndexedSeq<Tuple3<S, S, Number>> aux = Array.empty();
		return Stream.range(0, in.size()).foldLeft(aux,
				(currentI, i)->{
					return Stream.range(0, in.get(i)._1.length()).foldLeft(currentI, 
						(currentJ,j)->{
							return partOf(currentJ, in.get(i)._1.get(j), in.get(i)._2.get(j) );
						} );
					}  );
	}
	/**
	 * porovna znak se zpracovanymi znaky a spravne zaradi
	 * @param aux
	 * @param SAt
	 * @return
	 */
	private IndexedSeq<Tuple3<S, S, Number>> partOf(
												final IndexedSeq<Tuple3<S, S, Number>> aux,
												final S SAtA,
												final S SAtB) {
		if(aux.size() <1)
			return Array.of(new Tuple3<S, S, Number>(SAtA, SAtB, 1));
		final int index = getIndex(aux, SAtA, SAtB);
		return Stream.range(0, aux.size()).foldLeft(Array.empty(),
				(current, i)->{
					if(i == index){
						return current.append(
										new Tuple3<S, S, Number>(
												aux.get(i)._1,
												aux.get(i)._2,
												aux.get(i)._3.intValue()+1));
					}
					if(i == aux.size()-1 && index == -1 )
						return current
								.append(
										new Tuple3<S, S, Number>(
												aux.get(i)._1,
												aux.get(i)._2,
												aux.get(i)._3))
								.append(
										new Tuple3<S, S, Number>(
												SAtA,
												SAtB,
												1));
					return current.append(
										new Tuple3<S, S, Number>(
												aux.get(i)._1,
												aux.get(i)._2,
												aux.get(i)._3));
				});
	}
	
	/**
	 * projde sequenci a vrati index daneho znaku nebo -1, pokud znak v sekvenci neni
	 * @param aux
	 * @param SAt
	 * @return
	 */	
	private int getIndex(final IndexedSeq<Tuple3<S, S, Number>> aux,
			final S SAtA, final S SAtB){
		return getIndex(aux, SAtA, SAtB, 0);
	}

	private int getIndex(final IndexedSeq<Tuple3<S, S, Number>> aux,
							final S SAtA, final S SAtB,final int i) {
		if(aux.size()<=i)
			return -1;
		else if(aux.get(i)._1 == SAtA && aux.get(i)._2 == SAtB)
			return i;
		return getIndex(aux, SAtA, SAtB, i+1);
	}
	
	
	
	/******************************************************/
	/**
	 * pocet fonem v jazyce
	 * @param in
	 * @param second
	 * @return
	 */
	private IndexedSeq<Tuple2<S, Number>> calculateFonems(
				final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> in,
				//final IndexedSeq<Tuple3<IndexedSeq<S>, IndexedSeq<S>, Double>> in
				final boolean second) {
		final IndexedSeq<Tuple2<S, Number>> aux =Array.empty();
		return Stream.range(0, in.size()).foldLeft(aux,
				(currentI, i)->{
					return Stream.range(0, in.get(i)._1.length()).foldLeft(currentI,
							(currentJ, j)->{
								if(second)
									return partOf(currentJ,in.get(i)._2.get(j));
								else
									return partOf(currentJ, in.get(i)._1.get(j));
							} );
				});
	}

	/**
	 * porovna znak se zpracovanymi znaky a spravne zaradi
	 * @param aux
	 * @param SAt
	 * @return
	 */
	private IndexedSeq<Tuple2<S, Number>> partOf(final IndexedSeq<Tuple2<S, Number>> aux,
														  final S SAt) {
		if(aux.size() <1)
			return Array.of(new Tuple2<S, Number>(SAt, 1));
		final int index = getIndex(aux, SAt);
		return Stream.range(0, aux.size()).foldLeft(Array.empty(),
				(current, i)->{
					if(i == index){
						return current.append(new Tuple2<S, Number>(aux.get(i)._1, aux.get(i)._2.intValue()+1));
					}
					if(i == aux.size()-1 && index == -1 )
						return current.append(new Tuple2<S, Number>(aux.get(i)._1, aux.get(i)._2))
								.append(new Tuple2<S, Number>(SAt, 1));
					return current.append(new Tuple2<S, Number>(aux.get(i)._1, aux.get(i)._2));
				});
	}
	
	
	/**
	 * projde sequenci a vrati index daneho znaku nebo -1, pokud znak v sekvenci neni
	 * @param aux
	 * @param SAt
	 * @return
	 */	
	private int getIndex(final IndexedSeq<Tuple2<S, Number>> aux, final S SAt){
		return getIndex(aux, SAt, 0);
	}

	private int getIndex(final IndexedSeq<Tuple2<S, Number>> aux,final S SAt,final int i) {
		if(aux.size()<=i)
			return -1;
		else if(aux.get(i)._1.equals(SAt))
			return i;
		return getIndex(aux, SAt, i+1);
	}
}
