package metrics;



import java.util.function.Function;

import io.vavr.Tuple2;
import io.vavr.Tuple3;
import io.vavr.collection.IndexedSeq;
import metrics.gui.MatrixPrinterGI;
import metrics.gui.PrinterGI;
import metrics.gui.SequencePrinterGI;
import metrics.metric.HammingGIInfo;
import metrics.metric.JaroGIInfo;
import metrics.metric.JaroWinklerGIInfo;
import metrics.metric.LevenshteinGIInfo;
import metrics.metric.MetricGIInfo;
import metrics.metric.StructureMatrixGI;
import metrics.metric.StructureSequenceGI;
import metrics.metricquick.HammingGIQuick;
import metrics.metricquick.JaroGIQuick;
import metrics.metricquick.JaroWinklerGIQuick;
import metrics.metricquick.LevenshteinGIQuick;
import metrics.metricquick.MetricGIQuick;
import metrics.structures.CostAndInstedGI;
import metrics.structures.MetricTypeGI;

/**
 * 
 * @author Ondej Nmec
 *
 * @param <I> - hlavni sekvence
 * @param <S> - vstupni trida
 */
public abstract class AbstractMetricCollectionGI<I, S> {// I - obsah hlavni sekvence
		
	//pole metrik pro jOptionPane
	private static final MetricTypeGI[] metricList =  {
				 MetricTypeGI.LEVENSHTEIN,
				 MetricTypeGI.HAMMING,
				 MetricTypeGI.JARO,
				 MetricTypeGI.JARO_WINKLER
				};
	public static MetricTypeGI[] metricList(){
		return metricList;
	}
	
	public abstract MetricTypeGI getType();
	public abstract IndexedSeq<I> getMetrics();
	public abstract Number getTotalDistance();
	/**
	 * vytvory specialni vystum
	 * @return sekvence trojic: sekvence, sekvence vzdalenost
	 */
	public abstract IndexedSeq<Tuple3<IndexedSeq<S>, IndexedSeq<S>, Number>> createTriples();
	
	/**
	 * vytvory specialni vystup 
	 * @return sekvence dvojic: sekvence, sekvence
	 */
	public abstract IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> createDoubles();
	
	
	
	
	/**
	 * vybere patricnou metriku
	 * @param from
	 * @param to
	 * @param ci
	 * @param type
	 * @return
	 */
	protected MetricGIQuick<S> selectMetric(final IndexedSeq<S> from, final IndexedSeq<S> to,
			final MetricTypeGI type) {
		switch(type){
		case LEVENSHTEIN: return new LevenshteinGIQuick<S>(from, to);
		case HAMMING: 	return new HammingGIQuick<S>(from, to);
		case JARO: return new JaroGIQuick<S>(from, to);
		case JARO_WINKLER: return new JaroWinklerGIQuick<S>(from, to);
		}
		System.err.println("Nelze vybrat typ metriky: " + type);
		return null;
	}
	protected MetricGIInfo<?, S> selectMetric(final IndexedSeq<S> from, final IndexedSeq<S> to,
			final MetricTypeGI type, final CostAndInstedGI<S> ci, final Function<String, String> translator) {
		switch(type){
		case LEVENSHTEIN: return new LevenshteinGIInfo<S>(from, to, ci, translator);
		case HAMMING: 	return new HammingGIInfo<S>(from, to, ci, translator);
		case JARO: return new JaroGIInfo<S>(from, to, ci, translator);
		case JARO_WINKLER: return new JaroWinklerGIInfo<S>(from, to, ci, translator);
		}
		System.err.println("Nelze vybrat typ metriky: " + type);
		return null;
	}
	
	/**
	 * vybere spravny zpusob vykresleni
	 * @param met
	 * @return
	 */
	@SuppressWarnings("unchecked")
	protected PrinterGI<S> selectPrinter(final MetricGIInfo<?, S> met, final MetricTypeGI type){
		if(type == MetricTypeGI.LEVENSHTEIN || type == MetricTypeGI.JARO || type == MetricTypeGI.JARO_WINKLER)
			return new MatrixPrinterGI<>(  (StructureMatrixGI<?, S>) met);
		if(type==MetricTypeGI.HAMMING)
			return new SequencePrinterGI<>( (StructureSequenceGI<S>) met);
		System.err.println("Nelze vybrat printer");
		return null;
			
	}
	
	
	
	
	

}
