package metrics;

import metrics.gui.PrinterGI;

import java.util.function.Function;

import io.vavr.Tuple2;
import io.vavr.Tuple3;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import metrics.metric.MetricGIInfo;
import metrics.structures.CostAndInstedGI;
import metrics.structures.MetricError;
import metrics.structures.MetricTypeGI;

public class MetricCollectionGIInfo<S> extends AbstractMetricCollectionGI<Tuple2<MetricGIInfo<?, S>, PrinterGI<S>>, S>{
	
	private final IndexedSeq<Tuple2<MetricGIInfo<?, S>, PrinterGI<S>>> metrics;
	private final CostAndInstedGI<S> ci;
	private final MetricTypeGI type;
	private final Number totalDistance;
	private final Function<String, String> translator;
	
	@SuppressWarnings("unchecked")
	public MetricCollectionGIInfo(final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> input,
			final MetricTypeGI type) {
		
		this.type = type;
		this.ci = new CostAndInstedGI<S>(
				(a)->(S)new Object(), //deletion
				(a)->(S)new Object(), //insertion
				(a)->a, //transposition
				(a)->a, //substution
				1.0, //delete cost
				1.0, //insertion cost
				1.0, //transposition cost
				1.0); //substitution cost
		this.translator = (a)->a;
		this.metrics = useMetric(input);
		this.totalDistance = calculateTotalDistance();
	}
	
	public MetricCollectionGIInfo(final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> input,
			 final MetricTypeGI type,final CostAndInstedGI<S> ci, final Function<String, String> translator) {
		
		this.type = type;
		this.ci = ci;
		this.translator = translator;
		this.metrics = useMetric(input);
		this.totalDistance = calculateTotalDistance();
	}
	
	
	/************/
	@Override
	public IndexedSeq<Tuple2<MetricGIInfo<?, S>, PrinterGI<S> >> getMetrics() {
		return metrics;
	}
	public final CostAndInstedGI<S> getCi() {
		return ci;
	}
	@Override
	public final MetricTypeGI getType() {
		return type;
	}

	@Override
	public Number getTotalDistance() {
		return totalDistance;
	}

	/**********/
	
    
	/**
	 * spocita metriku a vytvory vystup
	 * @return
	 */
	private IndexedSeq<Tuple2<MetricGIInfo<?, S>, PrinterGI<S>>> useMetric(
								final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> input) {
		return Stream.range(0, input.size()).foldLeft(Array.empty(),
				(curr, i)->{
					final MetricGIInfo<?, S> metric = selectMetric(
													input.get(i)._1,
													input.get(i)._2,
													type, ci, translator);
					if(!metric.getError().equals(MetricError.OK))
						return curr;
					return curr.append(new Tuple2<MetricGIInfo<?, S>, PrinterGI<S>>(
							metric, selectPrinter(metric, type) ));
				} );
	}

	
	private Number calculateTotalDistance() {
		if(this.type == MetricTypeGI.JARO || this.type == MetricTypeGI.JARO_WINKLER)
			return Stream.range(0, metrics.size()).foldLeft(0.0,
					(curr, i)->curr.doubleValue() + metrics.get(i)._1.getDistance().orElse(0.0).doubleValue())
					/metrics.size();
		return Stream.range(0, metrics.size()).foldLeft(0.0,
				(curr, i)->curr.doubleValue() + metrics.get(i)._1.getDistance().orElse(0.0).doubleValue());
	}


	/****************************/

	@Override
	public IndexedSeq<Tuple3<IndexedSeq<S>, IndexedSeq<S>, Number>> createTriples() {
		return Stream.range(0, metrics.size()).foldLeft(Array.empty(),
				(current, i)->{
					if(metrics.get(i)._1.getError() == MetricError.OK)
						return current.append(
								new Tuple3<>(
										metrics.get(i)._1.getFinalFrom().get(),
										metrics.get(i)._1.getFinalTo().get(),
										metrics.get(i)._1.getDistance().get()
										));
					return current;//TODO erorova hlaska
				});
	}

	@Override
	public IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> createDoubles() {
		return Stream.range(0, metrics.size()).foldLeft(Array.empty(),
				(current, i)->{
					if(metrics.get(i)._1.getError() == MetricError.OK)
						return current.append(
								new Tuple2<>(
										metrics.get(i)._1.getFinalFrom().get(),
										metrics.get(i)._1.getFinalTo().get()
										));
					return current;//TODO erorova hlaska
				});
	}
	



}
