package metrics;

import io.vavr.Tuple2;
import io.vavr.Tuple3;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import metrics.metricquick.MetricGIQuick;
import metrics.structures.MetricError;
import metrics.structures.MetricTypeGI;

public class MetricCollectionGIQuick<S> extends AbstractMetricCollectionGI<MetricGIQuick<S>, S> {
	private final IndexedSeq<MetricGIQuick<S>> metrics;
	private final MetricTypeGI type;
	private final Number totalDistance;

	public MetricCollectionGIQuick(
			final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> input, final MetricTypeGI type) {
		
		this.type = type;
		this.metrics = useMetric(input);
		this.totalDistance = calculateTotalDistance();
	}
	
		
	/************/
	@Override
	public IndexedSeq<MetricGIQuick<S>> getMetrics() {	
		return metrics;
	} 

	@Override
	public final MetricTypeGI getType() {
		return type;
	}

	@Override
	public Number getTotalDistance() {
		return totalDistance;
	}
	
	
	/*******/

	/**
	 * spocita metriku a vytvory vystup
	 * @return
	 */
	private IndexedSeq<MetricGIQuick<S>> useMetric(
								final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> input) {
		return Stream.range(0, input.size()).foldLeft(Array.empty(),
				(curr, i)->{
					final MetricGIQuick<S> metric = selectMetric(
															input.get(i)._1, 
															input.get(i)._2,
															type);
					if(!metric.getError().equals(MetricError.OK))
						return curr;
					return curr.append(metric);
				} );
	}


	private Number calculateTotalDistance() {
		if(this.type == MetricTypeGI.JARO || this.type == MetricTypeGI.JARO_WINKLER)
			return Stream.range(0, metrics.size()).foldLeft(0.0,
					(curr, i)->curr.doubleValue() + metrics.get(i).getDistance().orElse(0.0).doubleValue())
					/metrics.size();
		return Stream.range(0, metrics.size()).foldLeft(0.0,
				(curr, i)->curr.doubleValue() + metrics.get(i).getDistance().orElse(0.0).doubleValue());
	}


	/********************************/

	@Override
	public IndexedSeq<Tuple3<IndexedSeq<S>, IndexedSeq<S>, Number>> createTriples() {
		return Stream.range(0, metrics.size()).foldLeft(Array.empty(),
				(current, i)->{
					if(metrics.get(i).getError() == MetricError.OK)
						return current.append(
								new Tuple3<>(
										metrics.get(i).getFrom().get(),
										metrics.get(i).getTo().get(),
										metrics.get(i).getDistance().get()
										));
					return current; //TODO erorova hlaska
				});
	}
	
	@Override
	public IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> createDoubles() {
		return Stream.range(0, metrics.size()).foldLeft(Array.empty(),
				(current, i)->{
					if(metrics.get(i).getError() == MetricError.OK)
						return current.append(
								new Tuple2<>(
										metrics.get(i).getFrom().get(),
										metrics.get(i).getTo().get()
										));
					return current;//TODO erorova hlaska
				});
	}
	

}
