package metrics.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridLayout;
import java.util.function.Function;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import metrics.gui.PrinterGI;
import io.vavr.Tuple2;
import io.vavr.collection.IndexedSeq;
import metrics.metric.StructureMatrixGI;
import metrics.structures.MetricError;

/**
 * vykresleni metrik se strukturou matice
 * NENI IMMUTABLE - jpanel
 * @author Ondej Nmec
 *
 */
public class MatrixPrinterGI<R, S> implements PrinterGI<S>{

	private final StructureMatrixGI<R, S> metric;
	
	public MatrixPrinterGI(final StructureMatrixGI<R, S> met) {
		this.metric = met;
	}
	
	@Override
	public JPanel getView(Font strucureFont, Font descFont,
			final Function<S, Component> fromSToComp) {
		JPanel out = new JPanel();
		out.setLayout(new BorderLayout());
		
		if(metric.getError()==MetricError.OK){
			//zobrazeni struktury
			JPanel graphic = new JPanel();
			graphic.setLayout(
					new GridLayout(
							metric.getStructure().get().getRowSize()+1,
							metric.getStructure().get().getColumnSize()+1
					));
		
			for(int row= 0; row<= metric.getFrom().get().length()+metric.getShift();row++){
				for(int column = 0; column<=metric.getTo().get().length()+metric.getShift();column++){
					//prvni je prazdny
					if( (column==0 && row ==0)  || (column==metric.getShift() && row ==0) || (column==0 && row == metric.getShift() )  ){
						JLabel a = new JLabel("X");
						a.setForeground(new Color(0x00ff00));
						a.setBackground(new Color(0x000fff));
						//a.setSize(10, 10);
						graphic.add(a);
						}
					//pismena ve sloupci
					if(column==0 && row>metric.getShift() ){
						Component a = fromSToComp.apply(
								metric.getFrom().get().get(row-1-metric.getShift()));
						a.setForeground(new Color(0x0000ff));
						graphic.add(a);
					}
					//pismena v radku
					if(row==0 && column>metric.getShift()){
						Component a = fromSToComp.apply(
								metric.getTo().get().get(column-1-metric.getShift() ));
						a.setForeground(new Color(0x0000ff));
						graphic.add(a);
					}
					//cisla
					if(row>0 && column>0){
						JLabel a = new JLabel(
							metric.getValue(metric.getStructure().get().get(row-1, column-1))
							.toString()
								
								);
						
						if(partOf(row-1, column-1,metric.getIndexes().get()  ))
							a.setForeground(new Color(0xff0000));
						else
							a.setForeground(new Color(0x000000));
						a.setFont(strucureFont);  //new Font("Arial",Font.PLAIN, 18)
						graphic.add(a);
					}
				}
			}
			out.add(graphic, BorderLayout.CENTER);
			//popisek
		
			JTextArea desc = new JTextArea();
			desc.setEditable(false);
			desc.setText(metric.getFormatedText().orElse("Nebyla vloena data"));
			desc.setFont(descFont);  //new Font("Arial", Font.PLAIN, 18)
		
			out.add(new JScrollPane(desc), BorderLayout.SOUTH);
		}
		return out;
	}
	
	
	private boolean partOf(int i, int j, IndexedSeq<Tuple2<Integer, Integer>> indexes) {
		for(int a = 0; a < indexes.size();a++){
			if(indexes.get(a)._1==i && indexes.get(a)._2 ==j)
				return true;
		}
		return false;
	}

}
