package metrics.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridLayout;
import java.util.function.Function;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import metrics.gui.PrinterGI;
import metrics.metric.StructureSequenceGI;
import metrics.structures.MetricError;

/**
 * vykresleni metrik se strukturou sekvence
 * NENI IMMUTABLE
 * @author Ondej Nmec
 *
 */
public class SequencePrinterGI<S>  implements PrinterGI<S>{
	
	private final StructureSequenceGI<S> metric;
	
	public SequencePrinterGI(final StructureSequenceGI<S> met) {
		this.metric = met;
	}
	
	@Override
	public JPanel getView(Font strucureFont, Font descFont,
			final Function<S, Component> fromSToComp) {
		JPanel out = new JPanel();
		out.setLayout(new BorderLayout());
		if(metric.getError()==MetricError.OK){
			//zobrazeni struktury
			JPanel graphic = new JPanel();
			graphic.setLayout(new GridLayout(3, metric.getFrom().get().length()  ));
			for(int j = 0; j<3;j++){
				for(int i = 0; i<metric.getFrom().get().length(); i++){
					Component c;
					switch (j) {
					case 0:
						c= fromSToComp.apply(metric.getFrom().get().get(i));
						break;
					case 1:
						c= fromSToComp.apply(metric.getTo().get().get(i));
						break;
					case 2:
						c = new JLabel(metric.getStructure().get().charAt(i)+"");
						break;
					default:
						c = new JLabel();
					}
					c.setFont(strucureFont);
					graphic.add(c);
				}
			}
			
			
			out.add(graphic, BorderLayout.CENTER);
			//popisek
		
			JTextArea desc = new JTextArea();
			desc.setEditable(false);
			desc.setText(metric.getFormatedText().get());
			desc.setFont(descFont); //new Font("Arial", Font.PLAIN, 18)
		
			out.add(new JScrollPane(desc), BorderLayout.SOUTH);
			
		}
		return out;
	}
	
	
}
