package metrics.metric;

import java.util.Optional;
import java.util.function.Function;

import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import metrics.structures.CostAndInstedGI;
import metrics.structures.MetricError;

public class HammingGIInfo<S> implements StructureSequenceGI<S> {

	private final IndexedSeq<S> from;
	private final IndexedSeq<S> to;
	private final IndexedSeq<S> finalFrom;
	private final IndexedSeq<S> finalTo;
	private final String output;
	private final CostAndInstedGI<S> ci;
	private final Integer distance;
	private final Integer weightDistance;
	private final String result; //ulozene hodnoty 
	private final MetricError error;
	
	private final Function<String, String> translator;
	
	public HammingGIInfo(final IndexedSeq<S> from, final IndexedSeq<S> to,
				   final CostAndInstedGI<S> ci, final Function<String, String> translator) {
		
		//uvodni kontrola
		if( (from == null || to == null)){
			this.from = null;
			this.to = null;
			this.ci = null;
			this.finalFrom = null;
			this.finalTo = null;
			this.output = null;
			this.result = null;	
			this.distance = null;
			this.weightDistance = null;
			this.translator = null;
			this.error = MetricError.NULL_DATA;
		}else if((from.length()<=0 && to.length()<0)){
			this.from = null;
			this.to = null;
			this.ci = null;
			this.finalFrom = null;
			this.finalTo = null;
			this.output = null;
			this.result = null;	
			this.distance = null;
			this.weightDistance = null;
			this.translator = null;
			this.error = MetricError.WRONG_DATA;
		}else if(from.length() != to.length() ){
			this.from = null;
			this.to = null;
			this.ci = null;
			this.finalFrom = null;
			this.finalTo = null;
			this.output = null;
			this.result = null;	
			this.distance = null;
			this.weightDistance = null;
			this.translator = null;
			this.error = MetricError.DIFERENT_LENGTH;
		}else{
			this.from = from;
			this.to = to;
			this.ci = ci;
			this.translator = translator;
			this.result = booleanSting(from, to);
			this.finalFrom = createResult(from, result);
			this.finalTo = createResult(to, result);
			this.distance = sum(result);
			this.weightDistance = sumWeight(result);
			this.output = createOutPrint(from, to, result);
			this.error = MetricError.OK;
		}
		
		
		
		
	}
/********************************************/
	
	@Override
	public Optional<IndexedSeq<S>> getFrom() {
		if(from == null)
			return Optional.empty();
		return Optional.of(from);
	}

	@Override
	public Optional<IndexedSeq<S>> getTo() {
		if(to == null)
			return Optional.empty();
		return Optional.of(to);
	}

	@Override
	public Optional<IndexedSeq<S>> getFinalFrom() {
		if(finalFrom == null)
			return Optional.empty();
		return Optional.of(finalFrom);
	}

	@Override
	public Optional<IndexedSeq<S>> getFinalTo() {
		if(finalTo == null)
			return Optional.empty();
		return Optional.of(finalTo);
	}

	@Override
	public Optional<String> getStructure() {
		if(result == null)
			return Optional.empty();
		return Optional.of(result);
	}

	@Override
	public Optional<Number> getDistance() {
		if(this.distance == null)
			return Optional.empty();
		return Optional.of(distance);
	}
	
	@Override
	public Optional<Number> getWeightDistance() {
		if(this.weightDistance == null)
			return Optional.empty();
		return Optional.of(weightDistance);
	}
	


	@Override
	public Optional<String> getFormatedText() {
		if(output == null)
			return Optional.empty();
		return Optional.of(output);
	}
	
	@Override
	public MetricError getError() {
		return error;
	}
	
	/**********************************/
	/**
	 * kontroluje podobnost jednotlivych znaku
	 * @param from
	 * @param to
	 * @return - string podobnosti - 1 stejne, 0 ruzne
	 */
	private String booleanSting(final IndexedSeq<S> from,final IndexedSeq<S> to) {
		return Stream.range(0, from.length()).foldLeft("",
				(current, i)->{
					if(from.get(i).equals(to.get(i)) )
						return current+ "0";
					return current+ "1";
				});
	}
	
	/**
	 * spocita vzdalenost ze stringu podobnosti
	 * @param booleanString
	 * @return
	 */
	private int sum(final String booleanString){
		return Stream.range(0, booleanString.length()).foldLeft(0, 
				(current, i)->{
					if(booleanString.charAt(i) == '1')
						return current +1;
					return current;
				});
	}
	
	private int sumWeight(final String booleanString){
		return Stream.range(0, booleanString.length()).foldLeft(0, 
				(current, i)->{
					if(booleanString.charAt(i) == '1')
						return current + (int)(1 * ci.getSubstitutionCost().intValue() ) ;
					return current;
				});
	}

	/**
	 * vytvory slovni vystup
	 * @param from
	 * @param to
	 * @param result
	 * @return
	 */
	private String createOutPrint(final IndexedSeq<S> from,final IndexedSeq<S> to,final String result) {
		return Stream.range(0, result.length()).foldLeft("",
				(current, i)->{
					if(result.charAt(i)=='1')
						return current
								+ this.translator.apply("substitution")  
								+ " '"//"Nhrada '" 
								+ from.get(i)
								+ "' "
								+ this.translator.apply("to") 
								+" '" 
								+ to.get(i)
								+ "' "
								+ this.translator.apply("position")
								+ " "
								+(i+1)+ "\n";
					return current;
				});
	}
	
	/**
	 * vytvory upravene dvojice
	 * @param origin - puvodni string
	 * @param bool - string podobnosti
	 * @return - upraveny string podle bool a ci
	 */
	private IndexedSeq<S> createResult(final IndexedSeq<S> origin, final String bool) {
		return Stream.range(0, origin.length()).foldLeft(Array.empty(),
				(curr, i)->{
					if(bool.charAt(i)=='1')
						return curr.append( ci.getSubstitution().apply(origin.get(i)) ) ;
					return curr.append( origin.get(i));  
				});
	}
	

}
