package metrics.metric;

import java.util.Optional;
import java.util.function.Function;

import io.vavr.Tuple2;
import io.vavr.Tuple4;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import metrics.structures.CostAndInstedGI;
import metrics.structures.MetricError;
import metrics.structures.MatrixGI;

public class LevenshteinGIInfo<S> 
      implements StructureMatrixGI<Tuple2<Integer, Boolean>, S>{

	private final IndexedSeq<S> from;
	private final IndexedSeq<S> to;
	private final IndexedSeq<S> finalFrom;
	private final IndexedSeq<S> finalTo;
	private final String output;
	private final MatrixGI<Tuple2<Integer, Boolean>> mat;
	private final CostAndInstedGI<S> ci;
	private final IndexedSeq<Tuple2<Integer, Integer>> indexes;
	private final Integer distance; 
	private final Integer weightDistance;
	private final MetricError error;

	private final Function<String, String> translator;
	
	public LevenshteinGIInfo(final IndexedSeq<S> from, final IndexedSeq<S> to, final CostAndInstedGI<S> ci,
			final Function<String, String> translator) {
		//uvodni kontrola
		if( (from == null || to == null) ){
			this.from = null;
			this.to = null;
			this.ci = null;
			this.mat = null;
			this.finalFrom = null;
			this.finalTo = null;
			this.output = null;
			this.indexes = null;
			this.weightDistance = null;
			this.distance = null;
			this.translator = null;
			this.error = MetricError.NULL_DATA;
		}else if((from.length()<=0 && to.length()<0)){
			this.from = null;
			this.to = null;
			this.ci = null;
			this.mat = null;
			this.finalFrom = null;
			this.finalTo = null;
			this.output = null;
			this.indexes = null;
			this.distance = null;
			this.weightDistance = null;
			this.translator = null;
			this.error = MetricError.WRONG_DATA;
		}else{
			this.translator = translator;
			this.from = from;
			this.to = to;
			this.ci = ci;

			
			@SuppressWarnings("unchecked")
			Tuple2<Integer, Boolean>[][] aux = new Tuple2[from.length()+1] [to.length()+1];
			
			
			this.mat = new MatrixGI<>(aux);
			
			final Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>>
				res = result(calculate(startFill(mat)));
			this.finalFrom = res._1;
			this.finalTo = res._2;
			this.indexes = res._3;
			this.distance = mat.get(mat.getRowSize()-1, mat.getColumnSize()-1)._1;
			this.weightDistance = calculateFinalDistance(res._4);
			this.output = createText(res._4);
			this.error = MetricError.OK;
		}
		
	}

	/********************************************/
	
	@Override
	public Optional<IndexedSeq<S>> getFrom() {
		if(from == null)
			return Optional.empty();
		return Optional.of(from);
	}

	@Override
	public Optional<IndexedSeq<S>> getTo() {
		if(to == null)
			return Optional.empty();
		return Optional.of(to);
	}

	@Override
	public Optional<IndexedSeq<S>> getFinalFrom() {
		if(finalFrom == null)
			return Optional.empty();
		return Optional.of(finalFrom);
	}

	@Override
	public Optional<IndexedSeq<S>> getFinalTo() {
		if(finalTo == null)
			return Optional.empty();
		return Optional.of(finalTo);
	}

	@Override
	public Optional<MatrixGI<Tuple2<Integer, Boolean>>> getStructure() {
		if(mat == null)
			return Optional.empty();
		return Optional.of(mat);
	}

	@Override
	public Optional<Number> getDistance() {
		if(distance == null)
			return Optional.empty();
		return Optional.of(distance);
	}
	
	@Override
	public Optional<Number> getWeightDistance() {
		if(weightDistance == null)
			return Optional.empty();
		return Optional.of(weightDistance);
	}

	@Override
	public Optional<String> getFormatedText() {
		if(output == null)
			return Optional.empty();
		return Optional.of(output);
	}


	@Override
	public Optional<IndexedSeq<Tuple2<Integer, Integer>>> getIndexes() {
		if(indexes == null)
			return Optional.empty();
		return Optional.of(indexes);
	}


	@Override
	public int getShift() {
		return 1;
	}


	@Override
	public String getValue(Tuple2<Integer, Boolean> origin) {
		return origin._1.toString();
	}

	@Override
	public MetricError getError() {
		return error;
	}
	
	/******************************************/
	
	
	/**
	 * priprava matice
	 * naplneni vychozimi hodnotami
	 * @param mat - vytvorena matice
	 * @return - nova matice
	 */
	private MatrixGI<Tuple2<Integer, Boolean>> startFill(final MatrixGI<Tuple2<Integer, Boolean>> mat){
		final MatrixGI<Tuple2<Integer, Boolean>> aux = mat;
		return ( Stream.range(0, aux.getRowSize()).foldLeft(aux,
				(row, i)->Stream.range(0, aux.getColumnSize()).foldLeft(aux,
						(column, j)->{
							if(j==0)
								return aux.withCell(i, j, new Tuple2<Integer, Boolean>(i, false));
							if(i==0)
								return aux.withCell(i, j, new Tuple2<Integer, Boolean>(j, false));
							System.out.println(from.get(i-1));
							System.out.println(to.get(j-1));
							System.out.println(from.get(i-1).equals(to.get(j-1)));
							System.out.println();
							boolean b = from.get(i-1).equals(to.get(j-1));
							return aux.withCell(i, j, new Tuple2<Integer, Boolean>(0,b));
						} ) ));
	}

	/**
	 * naplneni matice hodnotami
	 * @param mat - jiz hotova matice z startFill
	 * @return - nova matice
	 */
	private MatrixGI<Tuple2<Integer, Boolean>> calculate(final MatrixGI<Tuple2<Integer, Boolean>> mat){
		MatrixGI<Tuple2<Integer, Boolean>> aux = mat;
		return (Stream.range(1, aux.getRowSize()).foldLeft(aux,
				(row, i)->Stream.range(1, aux.getColumnSize()).foldLeft(aux,
						(column, j)->{
							boolean b = aux.get(i, j)._2;
							Integer value = Math.min( 
												aux.get(i-1, j)._1+1,
												Math.min( 
													aux.get(i, j-1)._1+1,
													aux.get(i-1, j-1)._1 + ((b)?0:1) ));
							return aux.withCell(i, j, new Tuple2<Integer, Boolean>(value,b));
						} ) ));
	}
	
	
	/**
	 * spusti prohledavani matice
	 * prazdne stringy - vysledna slova
	 * empty array - indexes
	 * 0, 0 - startovni bod prohledavani
	 * @return
	 */
	private Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>>
			result(final MatrixGI<Tuple2<Integer, Boolean>> mat){
		return result(mat,
				new Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>>
									(Array.empty(), Array.empty(), Array.empty(), Array.empty())
										, 0, 0);
	}
	
	/**
	 * upravy stringy na stejnou delku, vlozi pomocne znaky a vytvory seznam indexu
	 * @param strings - pomocna promena - stringy -> finalni stringy, sekvence -> indexes
	 * @param i - kolikaty radek
	 * @param j - kolikaty sloupec
	 * @return
	 */

	private Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>> 
			result(final MatrixGI<Tuple2<Integer, Boolean>> mat,
					final Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>> strings,
					final int i,final int j) {
		if(i<mat.getRowSize()-1 && j<mat.getColumnSize()-1){
			if(mat.get(i+1, j+1)._2)//shodne
				return result(mat,
						new Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>>(
								strings._1.append(from.get(i)),
								strings._2.append(to.get(j)),
								strings._3.append(new Tuple2<Integer, Integer>(i, j)),
								strings._4.append('E')
								)
						,i+1,j+1);
			final int direct = look(i+1,j+1);
			if(direct == 0)//smazani 
				//look(i+1,j+1,true)
				return result(mat,
						new Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>>(
								strings._1.append(from.get(i)),//pri prevodu jedno v druhe toto zde neni
								strings._2.append( ci.getDeletion().apply(' ') ) ,
								strings._3.append(new Tuple2<Integer, Integer>(i, j)),
								strings._4.append('D')
								),
						i+1,j);
			if(direct == 1)//vlozeni
				//look(i+1,j+1,false)
				return result(mat,
						new Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>>(
								strings._1.append(ci.getInsertion().apply(' ') ) ,
								strings._2.append(to.get(j)),//pri prevodu jedno v druhe toto zde neni
								strings._3.append(new Tuple2<Integer, Integer>(i, j)),
								strings._4.append('I')
								)
						
						,i,j+1);	
			return result(mat,              //substituce
					new Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>>(
							strings._1.append( ci.getSubstitution().apply(from.get(i)))  ,
							strings._2.append( ci.getSubstitution().apply(to.get(j)) ),
							strings._3.append(new Tuple2<Integer, Integer>(i, j)),
							strings._4.append('S')
							)
					, i+1,j+1);
		//na spodku matice, jen vkladani	
		}else if(i >= mat.getRowSize()-1 && j < mat.getColumnSize()-1){
			return result(mat,
					new Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>>(
							strings._1.append(ci.getInsertion().apply(' ')) ,
							strings._2.append( to.get(j)),//pri prevodu jedno v druhe toto zde neni
							strings._3.append(new Tuple2<Integer, Integer>(i, j)),
							strings._4.append('I')
							)
					,i,j+1);	
		//na boku matice, jen mazani	
		}else if(j >= mat.getColumnSize()-1 && i< mat.getRowSize()-1){
			return result(mat,
					new Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer, Integer>>, IndexedSeq<Character>>(
							strings._1.append(from.get(i)),//pri prevodu jedno v druhe toto zde neni
							strings._2.append( ci.getDeletion().apply(' ') ),
							strings._3.append(new Tuple2<Integer, Integer>(i, j)),
							strings._4.append('D')
							)
					,i+1,j);
		}	
			
		return new Tuple4<IndexedSeq<S>, IndexedSeq<S>, IndexedSeq<Tuple2<Integer,Integer>>, IndexedSeq<Character>>(
				strings._1, strings._2, strings._3.append(new Tuple2<Integer, Integer>(i, j)), strings._4.append('E'));
	}
	
	/**
	 * projde radek/sloupec/diagonalu a hleda prvek s cost 0
	 * @param i
	 * @param j
	 * @return 0 jit ve sloupci, 1 jit v radku, ostatni diagonala
	 */
	private int look(final int i,final int j){
		final int dia = distance(i, j, 2);
		final int col = distance(i, j, 1);
		final int row = distance(i, j, 0);
		
		if(col == -1 && row == -1)
			return -1;
		//smazani
		if(col != -1 && dia == -1)
			return 0;
		if(col != -1 && dia > col)
			return 0;
		//vlozeni
		if(row != -1 && dia == -1)
			return 1;
		if(row != -1 && dia > row)
			return 1;
		return -1;
	}
	
	/**
	 * projde radek/sloupec/diagonalu od zadaneho bodu a hleda bunku s hodnotou true
	 * @param i
	 * @param j
	 * @param mode - 0 radek, 1 sloupec, ostatni - diagonala
	 * @return vzdalenost k nejblizsimu, nebo -1
	 */
	// rozsah vyhledavni ??
	private int distance(final int i, final int j, final int mode){
	//	final int range = Math.min(from.length(), to.length())/2;
		if(mode == 0) //radek
			return Stream.range(j, mat.getColumnSize()).foldLeft(-1, 
					(curr, n)->{
						if(curr!=-1 && curr < mat.getColumnSize()) //range
							return curr;
						if(mat.get(i, n)._2)
							return n-j;
						return curr;
					});
		if(mode == 1) //sloupec
			return Stream.range(i, mat.getRowSize()).foldLeft(-1 , 
					(curr, n)->{
						if(curr!=-1 && curr < mat.getRowSize()) //  range
							return curr;
						if(mat.get(n, j)._2)
							return n-i;
						return curr;
					});	
		//diagonala
		return Stream.range(0, mat.getRowSize()).foldLeft(-1, 
				(curr, n)->{
					if(curr!=-1 && curr < mat.getRowSize()) //range
						return curr;
					if( i+n< mat.getRowSize() && j+n<mat.getColumnSize() &&
							mat.get(i+n, j+n)._2)
						return n;
					return curr;
				});
	}
	
	
	
	/**
	 * vytvory slovni vystup
	 * @return
	 */

	private String createText(final IndexedSeq<Character> operations) {
		return Stream.range(0, operations.length()).foldLeft("",
				(current, i)->{
					//insertion
					if(operations.get(i)=='I')
						return current
								+ this.translator.apply("insertion") 
								+ ": '"
								+ finalTo.get(i).toString()
								+ "' "
								+ this.translator.apply("position") 
								+ " "
								+ (i+1)
								+ "\n";
					//delete
					if(operations.get(i)=='D')
						return current
								+ this.translator.apply("deletion") 
								+ ": '"
								+ finalFrom.get(i).toString()
								+ "' "
								+ this.translator.apply("position") 
								+ " " 
								+ (i+1)
								+ "\n";
					//substituce
					if(operations.get(i)=='S')
						return current
								+ this.translator.apply("substitution") 
								+ ": '"
								+finalFrom.get(i).toString()
								+ "' "
								+ this.translator.apply("to") 
								+ " '"
								+ finalTo.get(i).toString()
								+ "' "
								+ this.translator.apply("position") 
								+ " "
								+ (i+1)
								+ "\n";
					return current;
				});
		
		
	}
	
	/**
	 * zpocita distance na zaklade naroku na cost
	 * @param seq
	 * @return
	 */
	private int calculateFinalDistance(final IndexedSeq<Character> seq){
		return Stream.range(0, seq.length()).foldLeft(0,
				(current, i)->{
					//insertion
					if(seq.get(i)=='I')
						return current + ci.getInsertionCost().intValue();
					//delete
					if(seq.get(i)=='D')
						return current + ci.getDeletionCost().intValue();
					//substituce
					if(seq.get(i)=='S')
						return current + ci.getSubstitutionCost().intValue();
					return current;
				} );
	}
	
	
	
	
}
