package metrics.metricquick;

import java.util.Optional;

import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import metrics.structures.MetricError;

public class HammingGIQuick<S> implements MetricGIQuick<S> {

	private final IndexedSeq<S> from;
	private final IndexedSeq<S> to;
	private final Integer distance; 
	private final MetricError error;
	
	public HammingGIQuick(final IndexedSeq<S> from, final IndexedSeq<S> to) {
		if( (from == null || to == null)){
			this.from = null;
			this.to = null;
			this.distance = null;
			this.error = MetricError.NULL_DATA;
		}else if((from.length()<=0 && to.length()<0)){
			this.from = null;
			this.to = null;
			this.distance = null;
			this.error = MetricError.WRONG_DATA;
		}else if(from.length() != to.length() ){
			this.from = null;
			this.to = null;
			this.distance = null;
			this.error = MetricError.DIFERENT_LENGTH;
		}else{
			this.from = from;
			this.to = to;
			this.distance = calculate(from, to);
			this.error= MetricError.OK;
		}
	}

	@Override
	public Optional<IndexedSeq<S>> getFrom() {
		if(from == null)
			return Optional.empty();
		return Optional.of(from);
	}

	@Override
	public Optional<IndexedSeq<S>> getTo() {
		if(to == null)
			return Optional.empty();
		return Optional.of(to);
	}

	@Override
	public Optional<Number> getDistance() {
		if(distance == null)
			return Optional.empty();
		return Optional.of(distance);
	}

	@Override
	public MetricError getError() {
		return error;
	}
	
	/**
	 * spocita vzdalenost
	 * @param from
	 * @param to
	 * @return
	 */
	private Integer calculate(final IndexedSeq<S> from,final IndexedSeq<S> to) {
		return Stream.range(0, from.length()).foldLeft(0,
				(current, i)->{
					if(from.get(i).equals(to.get(i)) )
						return current;
					return current+1;
				});
	}

	
	
	

}
