package metrics.metricquick;

import java.util.Optional;

import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import metrics.structures.MetricError;

public class JaroWinklerGIQuick<S> implements MetricGIQuick<S> {

	private final JaroGIQuick<S> jaro;
	private final Double distance; 
	private final MetricError error;
	
	public JaroWinklerGIQuick(final IndexedSeq<S> from, final IndexedSeq<S> to) {
		this.jaro = new JaroGIQuick<>(from, to);
		if( (from == null || to == null)){
			this.distance = null;
			this.error = MetricError.NULL_DATA;
		}else if((from.length()<=0 && to.length()<0)	){
			this.distance = null;
			this.error = MetricError.WRONG_DATA;
		}else{
			this.distance = calculateDistance(0.1, getAlfa(from, to));
			this.error = MetricError.OK;
		}
	}
	
	public JaroWinklerGIQuick(
			final IndexedSeq<S> from, final IndexedSeq<S> to, final double parameter) {
		this.jaro = new JaroGIQuick<>(from, to);
		this.distance = calculateDistance(parameter, getAlfa(from, to));
		this.error = null;
	}
	
	
	@Override
	public Optional<IndexedSeq<S>> getFrom() {
		return jaro.getFrom();
	}

	@Override
	public Optional<IndexedSeq<S>> getTo() {
		return jaro.getTo();
	}

	@Override
	public Optional<Number> getDistance() {
		if(distance == null)
			return Optional.empty();
		return Optional.of(distance);
	}

	@Override
	public MetricError getError() {
		return error;
	}
	
	
	/**
	 * vypocita vzdalenost na zaklade jaroovy metriky
	 * @param p
	 * @param alfa
	 * @return
	 */
	private double calculateDistance( final double p,final double alfa) {
		final Number jaroDistance = jaro.getDistance().orElse(0.0);
		return jaroDistance.doubleValue() + (alfa*p*(1-jaroDistance.doubleValue() ));
	}

	/**
	 * zjisteni alfa slozky - velikost spolecneho zakladu do velikosti 4
	 * @param from
	 * @param to
	 * @return
	 */
	private double getAlfa(final IndexedSeq<S> from,final  IndexedSeq<S> to) {
		return Stream.range(0, Math.min(4, Math.min(from.length(), to.length())))
				.foldLeft(0,
					(current, i)->{
						if(i>current)
							return current;
						if(from.get(i).equals( to.get(i) ) )
							return current +1;
						return current;
				});
	}

	
}
