package metrics.metricquick;

import java.util.Optional;

import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import metrics.structures.MetricError;
import metrics.structures.MatrixGI;

public class LevenshteinGIQuick<S> implements MetricGIQuick<S> {

	private final IndexedSeq<S> from;
	private final IndexedSeq<S> to;
	private final Integer distance; 
	private final MetricError error;
	
	public LevenshteinGIQuick(final IndexedSeq<S> from, final IndexedSeq<S> to) {
		if( (from == null || to == null) ){
			this.from = null;
			this.to = null;
			this.distance = null;
			this.error = MetricError.NULL_DATA;
		}else if((from.length()<=0 && to.length()<0)){
			this.from = null;
			this.to = null;
			this.distance = null;
			this.error = MetricError.WRONG_DATA;
		}else{
			this.from = from;
			this.to = to;
			this.distance = calculate(
								prepareMatrix(
										new MatrixGI<>(
												new Integer[from.length()+1][to.length()+1])));
			this.error = MetricError.OK;
		}
	}
	
	@Override
	public Optional<IndexedSeq<S>> getFrom() {
		if(from == null)
			return Optional.empty();
		return Optional.of(from);
	}

	@Override
	public Optional<IndexedSeq<S>> getTo() {
		if(to == null)
			return Optional.empty();
		return Optional.of(to);
	}

	@Override
	public Optional<Number> getDistance() {
		if(distance == null)
			return Optional.empty();
		return Optional.of(distance);
	}

	@Override
	public MetricError getError() {
		return error;
	}
	
	/**
	 * pripravi matici na vypocet
	 * @param mat
	 * @return
	 */
	private MatrixGI<Integer> prepareMatrix(final MatrixGI<Integer> mat){
		return Stream.range(0, mat.getRowSize()).foldLeft(mat,
				(row, n)->Stream.range(0, mat.getColumnSize()).foldLeft(mat,
						(column, m)->{
							if(n==0 && m==0)
								return mat.withCell(n, m, 0);
							if(n==0)
								return mat.withCell(n, m, m);
							if(m==0)
								return mat.withCell(n, m, n);
							return mat.withCell(n, m, 
									Math.min(
											Math.min(
													mat.get(n-1, m-1)+
														( (from.get(n-1).equals(to.get(m-1)) ) ? 0 : 1),
													mat.get(n, m-1) + 1),
											mat.get(n-1, m) + 1)
									
									);
						}));	
	} 
	
	/**
	 * vrati vyslednou vzdalenost
	 * @param mat
	 * @return
	 */
	private Integer calculate(final MatrixGI<Integer> mat){
		return mat.get(mat.getRowSize()-1, mat.getColumnSize()-1);
	}

	

}
