package gui;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.Locale;
import java.util.Optional;
import java.util.function.Function;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;

import entrophy.EntrophyGI;
import gui.viewer.GuiLoader;
import io.vavr.Tuple2;
import io.vavr.collection.IndexedSeq;
import lang.Lang;
import metrics.AbstractMetricCollectionGI;
import metrics.MetricCollectionGIInfo;
import metrics.MetricCollectionGIQuick;
import metrics.structures.CostAndInstedGI;
import metrics.structures.MetricTypeGI;
import rw.ReadWrite;
import structure.Structure;
import tbl.model.CalculateData;
import tbl.model.EntrophyData;
import tbl.model.RawData;

import javax.swing.AbstractAction;

public class ActionsLoader<S> {
	private Icon info;
	private Icon quick;
	private Icon entrophy;
	private Icon detail;
	private Icon load;
	private Icon save;
	private Icon file;
	private Icon help;
	private Icon about;
	
	private Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> input;
	private Optional<MetricCollectionGIInfo<S>> mc;
	private Main<S> main; 
	private final Structure<S> structure;
	
	public ActionsLoader(Main<S> main, final Structure<S> structure) {
		this.input = Optional.empty();
		this.mc = Optional.empty();
		this.main = main;
		this.structure = structure;
		//*
		this. info = new ImageIcon(getClass().getResource("/icon/icoInfo.png"));
		this.quick = new ImageIcon(getClass().getResource("/icon/icoQuick.png"));
		this.entrophy = new ImageIcon(getClass().getResource("/icon/icoEntrophy.png"));
		this.detail = new ImageIcon(getClass().getResource("/icon/icoDetail.png"));
		this.load = new ImageIcon(getClass().getResource("/icon/icoLoad.png"));
		this.save = new ImageIcon(getClass().getResource("/icon/icoSave.png"));
		this.file = new ImageIcon(getClass().getResource("/icon/icoFile.png"));
		this.help = new ImageIcon(getClass().getResource("/icon/icoHelp.png"));
		this.about = new ImageIcon(getClass().getResource("/icon/icoAbout.png"));
		//*/
		/*
		this. info = new ImageIcon("res/icon/icoInfo.png");
		this.quick = new ImageIcon("res/icon/icoQuick.png");
		this.entrophy = new ImageIcon("res/icon/icoEntrophy.png");
		this.detail = new ImageIcon("res/icon/icoDetail.png");
		this.load = new ImageIcon("res/icon/icoLoad.png");
		this.save = new ImageIcon("res/icon/icoSave.png");
		this.file = new ImageIcon("res/icon/icoFile.png");
		this.help = new ImageIcon("res/icon/icoHelp.png");
		this.about = new ImageIcon("res/icon/icoAbout.png");
		//*/
		/*
		this. info = new ImageIcon("icon/icoInfo.png");
		this.quick = new ImageIcon("icon/icoQuick.png");
		this.entrophy = new ImageIcon("icon/icoEntrophy.png");
		this.detail = new ImageIcon("icon/icoDetail.png");
		this.load = new ImageIcon("icon/icoLoad.png");
		this.save = new ImageIcon("icon/icoSave.png");
		this.file = new ImageIcon("icon/icoFile.png");
		this.help = new ImageIcon("icon/icoHelp.png");
		this.about = new ImageIcon("icon/icoAbout.png");
		//*/
	}
	
	/*************** info metriky ****************************/
	public Action actCalculateInfo(){
		Action act = new AbstractAction(//"Detailnejsi vypocet",
				Lang.getAction("action.calculateInfo.title"),
				info) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, selectMetric(), false, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Vypocet vzdalenosti s detailem"
										Lang.getAction("action.calculateInfo.desc"));
		return act;
	}
	
	public Action actInfoLevenshtein(){
		Action act = new AbstractAction(//"Levenstejnova vzdalenost"
									Lang.getAction("action.levenshteinInfo.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, MetricTypeGI.LEVENSHTEIN, false, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");
		return act;
	}
	
	public Action actInfoHamming(){
		Action act = new AbstractAction(//"Hammingova vzdalenost"
							Lang.getAction("action.hammingInfo.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, MetricTypeGI.HAMMING, false, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");		
		return act;
	}
	
	public Action actInfoJaro(){
		Action act = new AbstractAction(//"Jaroova vzdalenost"
				Lang.getAction("action.jaroInfo.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, MetricTypeGI.JARO, false, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");	
		return act;
	}
	
	public Action actInfoJaroWinkler(){
		Action act = new AbstractAction(//"Jaro-Winklerova vzdalenost"
				Lang.getAction("action.jarowinklerInfo.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, MetricTypeGI.JARO_WINKLER, false, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");	
		return act;
	}
	
	/*************** quick metrik *******************************/
	
	public Action actCalculateQuick(){
		Action act = new AbstractAction(//"Rychlejsi vypocet"
				Lang.getAction("action.calculateQuick.title"),
				quick) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, selectMetric(), true, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Vypocet vzdalenosti bez dalsich informaci"
						Lang.getAction("action.calculateQuick.desc"));	
		return act;
	}
	
	public Action actQuickLevenshtein(){
		Action act = new AbstractAction(//"Levenstejnova vzdalenost"
						Lang.getAction("action.levenshteinQuick.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, MetricTypeGI.LEVENSHTEIN, true, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");	
		return act;
	}
	
	public Action actQuickHamming(){
		Action act = new AbstractAction(//"Hammingova vzdalenost"
				Lang.getAction("action.hammingQuick.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, MetricTypeGI.HAMMING, true, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");	
		return act;
	}
	
	public Action actQuickJaro(){
		Action act = new AbstractAction(//"Jaroova vzdalenost"
				Lang.getAction("action.jaroQuick.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, MetricTypeGI.JARO, true, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");	
		return act;
	}
	
	
	public Action actQuickJaroWinkler(){
		Action act = new AbstractAction(//"Jaro-Winklerova vzdalenost"
				Lang.getAction("action.jarowinklerQuick.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				calculate(input, MetricTypeGI.JARO_WINKLER, true, structure.getCI());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");	
		return act;
	}
	
	/************* dalsi k metrikam ***********************/
	
	public Action actDetail(final Font matricViewFont){
		Action act = new AbstractAction(//"Zobrazit detail"
				Lang.getAction("action.showDetail.title"),
				detail) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				showDetail(main.getSelectedFrame(), matricViewFont);
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Zobrazi detail aktualni vybraneho radku, pokud je to mozne"
				Lang.getAction("action.showDetail.desc"));	
		return act;
	}
	
	
	public Action actEntrophy(){
		Action act = new AbstractAction(//"Spocitat entropii"
				Lang.getAction("action.calculateEntrophy.title"),
				entrophy) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				entrophy(structure.removeAuxChar());
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Spocita entropii z naposled spocitane metriky"
				Lang.getAction("action.calculateEntrophy.desc"));	
		return act;
	}
	
	/******************** soubory  *******************************/
	
	public Action actLoad(){
		Action act = new AbstractAction(//"Nacist data"
				Lang.getAction("action.loadData.title"),
				load) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				load(3);
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, 
				KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Umoznuje vyber zpusobu nacteni dat"
							Lang.getAction("action.loadData.desc"));	
		return act;
	}
	
	public Action actEdit(){
		Action act = new AbstractAction(//"Upravit data"
								Lang.getAction("action.editData.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				edit();
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Uprava vstupni data"
						Lang.getAction("action.editData.desc"));	
		return act;
	}
	
	public Action actLoadGui(){
		Action act = new AbstractAction(//"Nacist z GUI"
				Lang.getAction("action.loadDataGui.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				load(1);
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, 
				KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_MASK));
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Primo zadani dat"
						Lang.getAction("action.loadData.desc"));	
		return act;
	}
	
	public Action actLoadFile(){
		Action act = new AbstractAction(//"Nacist soubor"
				Lang.getAction("action.loadDataFile.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				load(0);
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, 
				KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Nacteni dat ze souboru"
				Lang.getAction("action.loadDataFile.desc"));	
		return act;
	}
	
	public Action actSave(){
		Action act = new AbstractAction(//"Ulozit"
				Lang.getAction("action.saveData.title"),
				save) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				save();
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, 
				KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Ulozeni aktualne vybraneho okna"
				Lang.getAction("action.saveData.desc"));	
		return act;
	}
	
	/***************************  program **************************/
	
	public Action actHelp(){
		Action act = new AbstractAction(//"Napoveda"
				Lang.getAction("action.help.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				help();
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, 
				KeyStroke.getKeyStroke(KeyEvent.VK_F1, InputEvent.ALT_MASK));
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Napoveda"
				Lang.getAction("action.help.desc"));	
		return act;
	}
	
	
	public Action actAbout(){
		Action act = new AbstractAction(//"O programu"
				Lang.getAction("action.about.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				about();
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY,
				KeyStroke.getKeyStroke(KeyEvent.VK_F2, InputEvent.ALT_MASK));
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Informace o programu"
				Lang.getAction("action.about.desc"));	
		return act;
	}
	
	public Action actClose(){
		Action act = new AbstractAction(//"Konec"
				Lang.getAction("action.close.title")) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				//"Ukonceni programu"
				main.log(Lang.getLog("log.action.closing"));
				System.exit(0);
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, 
				KeyStroke.getKeyStroke(KeyEvent.VK_F4, InputEvent.ALT_MASK));
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Ukonce program"
				Lang.getAction("action.close.desc"));	
		return act;
	}
	
	
	public Action actShowInput(){
		Action act = new AbstractAction(//"Zobrazit vstup"
				Lang.getAction("action.showInput.title"),
				file) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				showInput();
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,//"Zobrazi nactena data"
				Lang.getAction("action.showInput.desc"));	
		return act;
	}
	/*
	public Action actSetting(){
		Action act = new AbstractAction(
				Lang.getAction("action.setting.title")
				) {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				 showSetting();				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");	
		return act;
	}
	*/
	
	/********************************************************************************/
	
	/**
	 * vypocita metriku
	 * @param input
	 * @param type
	 * @param quick
	 */
	private void calculate(
			final Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> input,
			final MetricTypeGI type, final boolean quick,
			final CostAndInstedGI<S> ci){
		if(!input.equals(Optional.empty())){
			if(type!=null){
				//"Vybrana metrika: "
				main.log(Lang.getLog("log.action.selectedMetric") + type);
				Icon ico = (quick)?this.quick:this.info;
				AbstractMetricCollectionGI<?, S> aux = selectMetricCollection(input, type, quick, ci);
				main.addFrame(type.toString() + " " + aux.getTotalDistance(),
						new CalculateData<>(aux.createTriples()), selectInterName(quick),
						ico						
						);
				//"Spocitano"
				main.log(Lang.getLog("log.action.calculated"));
			}else
				main.log(Lang.getLog("log.action.noSelectedMetric"));  //"Nebyla vybrana metrika"
		}else
			main.log(Lang.getLog("log.action.noLoadData"));//	"Nejsou nactena data"	
	}
	
	/**
	 * vybere spravne intername
	 * @param quick
	 * @return
	 */
	private InternalName selectInterName(final boolean quick){
		if(quick)
			return InternalName.QUICK;
		return InternalName.INFO;
	}
	
	/**
	 * vytvory spravnou abstractcollecton
	 * @param type
	 * @param quick
	 * @return
	 */
	private AbstractMetricCollectionGI<?, S> selectMetricCollection(
			final Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> input,
			final MetricTypeGI type, final boolean quick,
			final CostAndInstedGI<S> ci){
		if(quick)
			return new MetricCollectionGIQuick<>(input.get(), type);
		MetricCollectionGIInfo<S> aux = new MetricCollectionGIInfo<>(
										input.get(),
										type,
										ci,
										(a)->Lang.getDetail(a));
		mc = Optional.of(aux);
		return aux;
	}
	
	/**
	 * zobrazi nabidku metrik uzivately
	 * @return typ metriky
	 */
	private MetricTypeGI selectMetric(){
		return (MetricTypeGI)JOptionPane.showInputDialog(
				null,
				//"Zvolte metriku:"
				Lang.getAction("action.selectMetric.message"),
				//"Vyber metrik"
				Lang.getAction("action.selectMetric.title"),
				JOptionPane.QUESTION_MESSAGE,
				null,
				AbstractMetricCollectionGI.metricList(),
				null
				);
	}

	/****************************************/
	/**
	 * zobrazi vstup
	 */
	private void showInput(){
		if(!input.equals(Optional.empty()))
			main.addFrame(
					//"Vstupni data"
					Lang.getAction("action.frameName.inputData"),
					new RawData<>(input.get(), false),
					InternalName.INPUT, load );
		else
			main.log(Lang.getLog("log.action.noLoadData"));  //"Zadna data nejsou nactena"
	}
	
	/**
	 * otazka na prepsani dat
	 * @param stringS
	 * @param option
	 */
	private void load(final int option){
		if(!input.equals(Optional.empty())){
			String[] yesNoOptions = {
					Lang.getAction("action.rewriteInput.optionYes"),
					Lang.getAction("action.rewriteInput.optionNo")};
			if(main.questionDialog(
					// "Pri nacteni novych dat budou stara data ztracena. Prejete si nacist nova?",
					Lang.getAction("action.rewriteInput.message"),
					//"Prepsat data",
					 Lang.getAction("action.rewriteInput.title"),
					yesNoOptions
					) == 0
					){
				nullate();
				input = makeInput(option);
				showInput();
			//	main.log("Nacteno");
			}
		}else{
			input = makeInput(option);
			showInput();
		//	main.log("Nacteno");
		}
	}
	
	/**
	 * editace nactenych dat
	 * @param stringS
	 */
	private void edit(){
		input = editData();
		if(!input.equals(Optional.empty())){
			Internal inter = main.getInternalByName(InternalName.INPUT).get();
			inter.setTableModel(new RawData<>(input.get(), false) );
		}
	}
	
	/**
	 * vybere zpusob nacitani
	 * @param option 0 - soubor, 1 - gui, 2 - nic, 3 - otazka
	 * @param stringS
	 * @return
	 */
	private Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> makeInput(
			final int option){
		switch (option) {
		case 0:
			main.log(Lang.getLog("log.action.loadFile"));  //"Nacitani ze souboru..."
			return loadDataFromFile();
		case 1:
			main.log(Lang.getLog("log.action.loadGui")); //"Nacitani z rozhrani"
			return loadDataFromGui();
		case 2:
			return Optional.empty();
		case 3:
			return loadDataWithMessage();	
		default:
			return Optional.empty();
		}
	}
	
	/**
	 * nacteni dat s dotazem na zdroj
	 * @param stringS
	 * @return
	 */
	private Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> loadDataWithMessage(){
		String[] options = {
				Lang.getAction("action.selectLoad.optionFile"),
				Lang.getAction("action.selectLoad.optionGui"),
				Lang.getAction("action.selectLoad.optionClose")};
		int opt = main.questionDialog(
				//"Chcete nacist data ze souboru nebo z rozhrani?",
				Lang.getAction("action.selectLoad.message"),
				//"Nacteni souboru",
				Lang.getAction("action.selectLoad.title"),
				options);
		return makeInput(opt);
	}
	/**
	 * nacte data ze soubouru
	 * @param stringS
	 * @return
	 */
	private Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> loadDataFromFile(){
		return new FileOperation<>(main, structure).load();
	}
	
	/**
	 * nacte data z gui
	 * @return
	 */
	private Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> loadDataFromGui(){
		return new GuiLoader<S>(structure.getEditor(), structure.getRenderer())
							.load();
	}
	
	/**
	 * zobrazi data v gui a umoznuje editaci
	 * @param stringS
	 * @return
	 */
	private Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> editData(){
		if(!input.equals(Optional.empty()))
			return new GuiLoader<S>(structure.getEditor(), structure.getRenderer())
							.edit(input.get());
		return Optional.empty();
	}
	
	/**
	 * smaze predesle zaznamy, vycisti pracovni plochu
	 */
	private void nullate(){
		main.log(Lang.getLog("log.action.deletingData"));  //"Mazu predesla data"
		//vse do null
		input=Optional.empty();
		mc = Optional.empty();
		
		main.clearDescTop();
		main.log(Lang.getLog("log.action.deleted")); //"Smazano"
	}
	
	/*********/
	
	@SuppressWarnings("unchecked")
	private void save(){
		Internal inter = main.getSelectedFrame();
		FileOperation<S> fo = new FileOperation<>(main, structure);
		if(inter != null){
			main.log(Lang.getLog("log.action.saving")); // "Ukladam data..."
			switch (inter.getInternalName()) {
			case INPUT:
				if(fo.save(input.get()))
					main.log(Lang.getLog("log.action.save"));  //"Ulo�eno"
				else
					main.log(Lang.getLog("log.action.noSave"));//"Nepodarilo se ulozit"
				
				break;
			case INFO:
				if(fo.save( (CalculateData<S>)inter.getTableModel().get() )  )
					main.log(Lang.getLog("log.action.save"));  //"Ulozeno"
				else
					main.log(Lang.getLog("log.action.noSave"));//"Nepodarilo se ulozit"
				break;
			case QUICK:
				if(fo.save( (CalculateData<S>)inter.getTableModel().get() ))
					main.log(Lang.getLog("log.action.save"));  //"Ulozeno"
				else
					main.log(Lang.getLog("log.action.noSave"));//"Nepodarilo se ulozit"
				break;
			case DETAIL:
				main.log(Lang.getLog("log.action.detailNoSave")); //"Detail nelze ulo�it"
				break;
			case ENTROPHY:
				if(fo.save((EntrophyData<S>)inter.getTableModel().get() ))
					main.log(Lang.getLog("log.action.save"));  //"Ulozeno"
				else
					main.log(Lang.getLog("log.action.noSave")); //"Nepodarilo se ulozit nektere ze souboru"
				;
				break;
				
			default:
				main.log(Lang.getLog("log.action.cantSave"));  //"Nelze ulozit"
				break;
			}
		}else{
			main.log(Lang.getLog("log.action.nothingToSave"));  //"Neni co ulozit"
		}
	}
	
	/********************************/
	/**
	 * inicializuje zobrazeni detailu metriky
	 * @param inter
	 * @param matricViewFont
	 */
	private void showDetail(final Internal inter, final Font matricViewFont){
		if( inter!=null && inter.getInternalName() == InternalName.INFO ){
			detail(inter.getSelectedRow(),mc,matricViewFont);
		}else{
			main.log(Lang.getLog("log.action.cantShowDetail"));  //"Nelze zobrazit detail"
		}
	}
	
	/**
	 * zobrazi detail metriky
	 * @param i
	 */
	private void detail(final int i, final Optional<MetricCollectionGIInfo<S>> mc, final Font matricViewFont){
		if(i == -1){
			main.log(Lang.getLog("log.action.noSelectedRow"));  //"Nebyl vybran zadny radek"
		}else if(mc.equals(Optional.empty()) ){
			main.log(Lang.getLog("log.action.noCalculatedData"));  //"Pro detail nebyla spocitana zadna data"
		}else if(i>= mc.get().getMetrics().size() ){
			main.log(Lang.getLog("log.action.cantShowDetail"));  //"Nelze zobrazit detail"
			System.err.println("Nelze zobrazit detail " + i + "/" + mc.get().getMetrics().size());
		}else{
			main.addFrame(//"Detail " 
					Lang.getAction("action.frameName.detail") + " " + i + " | " 
							+ mc.get().getMetrics().get(i)._1.getDistance().get(),
					new JScrollPane( mc.get().getMetrics().get(i)._2
							.getView(matricViewFont, matricViewFont, structure.fromSToComp())),
					detail);
			
		}
	}
	
	
	private void entrophy(final Function<Tuple2<IndexedSeq<S>, IndexedSeq<S>>, Tuple2<IndexedSeq<S>, IndexedSeq<S>>> remChar){
		if(!mc.equals(Optional.empty())  ){
			main.log(Lang.getLog("log.action.calculatingEntrophy")); //"Pocitam entropii..."
			EntrophyGI<S> ent = new EntrophyGI<>(
					mc.get().createDoubles(),
					remChar);
			main.addFrame(//"Entropie"
					Lang.getAction("action.frameName.entrophy")
					+ " | " + 
					ent.getEntropyFrom()
					+ " | " +
					ent.getEntropyTo(),
					new EntrophyData<>(ent),
					InternalName.ENTROPHY, entrophy);
			main.log(Lang.getLog("log.action.entrophyCalculated"));//"Entropie spocitana"
		}else{
			main.log(Lang.getLog("log.action.cantCalEntrophy"));//"Pro pocitani entropie nejsou spocitana data"
		}

	}
	
	/***************************************************************/
	private void about(){
		showOptionPane(
				getTextFrom("html/" + Locale.getDefault() + "/about.html"), //  "+Main.lang().name()+"/
				//"O programu"
				Lang.getAction("action.about.title"), about);
	}
	
	private void help(){
		showOptionPane(
				getTextFrom("html/" + Locale.getDefault() + "/help.html"),  //  "+Main.lang().name()+"/
				//"Napoveda",
				Lang.getAction("action.help.title"),help);
	}
	
	
	
	
	private String getTextFrom(final String fileName){
		ReadWrite rw = new ReadWrite();
		Optional<String> s = rw.getLoader().readAsOneString(
												///fileName,
												getClass().getResource("/"+fileName), 
											//	"res/" + fileName, 
											//		fileName,
												rw.getUnicodeCharset());
		if(s.equals(Optional.empty()))
			return "";
		return s.get();
	}
	/*
	private void showSetting(){
		String[] languages = {"etina", "english"};
		
		String opt = (String)JOptionPane.showInputDialog(
				null,
				Lang.getAction("action.selectLang.message"), 
				Lang.getAction("action.selectLang.title"),  
				JOptionPane.QUESTION_MESSAGE,
				null,
				languages,
				null
				);
		if(opt!=null)
			switch (opt) {
			case "etina":
				Locale.setDefault(new Locale("cs", "CZ"));
				break;
			case "english":
				//Locale.setDefault(Locale.UK);
				Locale.setDefault(new Locale("en"));
				break;
			default:
				break;
			}
		
	}
	*/
	private void showOptionPane(final String text, final String title, final Icon icon){
		//	JOptionPane.showMessageDialog(main, new JLabel(text));
		JOptionPane.showMessageDialog(main, 
				new JScrollPane(new JLabel(text)),
				 title, JOptionPane.INFORMATION_MESSAGE, icon);
	}
	
	
	/*
	public Action (){
		Action act = new AbstractAction() {
			private static final long serialVersionUID = 1L;
			@Override
			public void actionPerformed(ActionEvent e) {
				 Auto-generated method stub
				
			}
		};
		//klavesova zkratka
		act.putValue(AbstractAction.ACCELERATOR_KEY, null);
		//bublinova napoveda
		act.putValue(AbstractAction.SHORT_DESCRIPTION,"");	
		return act;
	}
	*/
}
