package gui;


import java.util.Optional;

import chooser.Chooser;
import io.vavr.Tuple2;
import io.vavr.Tuple3;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import lang.Lang;
import output.WriteMode;
import rw.ReadWrite;
import structure.Structure;
import tbl.model.CalculateData;
import tbl.model.EntrophyData;
import tbl.reworker.PlainTextReworker;
import tbl.reworker.XmlReworker;

/**
 * nacte data a upravy pro zpracovani
 * pripravy data na ukladani a ulozi
 * @author Ond�ej N�mec
 *
 * @param <S>
 */
public class FileOperation<S> {
	private final Main<S> main;
	private final ReadWrite rw;
	private final String path = "C:/Users/"+System.getenv("USERNAME")+"/Desktop/";  //soubory/
	private final IndexedSeq<Tuple2<String, String>> filters;
	private final PlainTextReworker<S> plaintext;
	private final XmlReworker<S> xml;

	public FileOperation(final Main<S> main, final Structure<S> struc) {
		this.rw = new ReadWrite();
		this.main = main;
		this.filters = prepareFilters();
		this.plaintext = new PlainTextReworker<>(
				struc.fromStringToSeq(),
				struc.fromSeqToString());
		this.xml = new XmlReworker<>(
				struc.fromSeqToObject(),
				struc.fromSToObject(),
				struc.fromObjectToSeq());
	}
	/**********************************/
	
	public Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> load(){
		final Chooser chooser = new Chooser(main, path, filters, true);
		if(!chooser.getFile().equals(Optional.empty()) ){
			//"Nacitan dokument "
			main.log(Lang.getLog("log.fileOp.loading")
					+ chooser.getFile().get().getName() + "...");
			return switchLoader(chooser);
		}
		main.log(Lang.getLog("log.fileOp.notLoad"));//"Soubor nenacten"
		return Optional.empty();
	}
	
	
	private Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> switchLoader(final Chooser chooser) {
		switch (chooser.getFile().get().getExtension().toLowerCase()) {
		case "csv":
			return plaintext.reworkInput(
					rw.getLoader().readGrid(
							chooser.getFile().get().getPath(), 
							rw.getWindowsCharset(),
							";"));
		case "txt":
			return plaintext.reworkInput(
					rw.getLoader().readGrid(
							chooser.getFile().get().getPath(), 
							rw.getWindowsCharset(),
							"\t"));
		case "xml":
			return xml.reworkInput(
					rw.getLoader().readXml(
							chooser.getFile().get().getPath(), 
							rw.getWindowsCharset()));

		default:
			return Optional.empty();
		}
	}
	/*********************************************************/
	public boolean save(final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> input){
		final Chooser chooser = new Chooser(main, path, filters, false);
		if(!chooser.getFile().equals(Optional.empty()) ){
			//"Ukladam dokument "
			main.log(Lang.getLog("log.fileOp.saving") 
					+ chooser.getFile().get().getName() + "...");
			return originCretor(
					chooser,
					input);
		}
		return false;
	}

		
	public boolean save(final CalculateData<S> calculate){
		final Chooser chooser = new Chooser(main, path, filters, false);
		if(!chooser.getFile().equals(Optional.empty()) ){
			//"Ukladam dokument "
			main.log(Lang.getLog("log.fileOp.saving")
					+ chooser.getFile().get().getName() + "...");
			return calculateCreator(
					chooser,
					calculate.getTriples());
		}
		return false;
	}
	
	public boolean save(final EntrophyData<S> entrophy) {
		final Chooser chooser = new Chooser(main, path, filters, false);
		if(!chooser.getFile().equals(Optional.empty()) ){
			boolean aux= false;
			main.log(Lang.getLog("log.fileOp.savingNumber") + " 1 ");//"Uladam 1. soubor"
			aux = eOveralCreator(
					chooser.getFile().get().getExtension(), chooser.getFile().get()
										.toString(//"_celkovy-vystup"
												Lang.getMessage("fileOp.entrophy.overalName")),
					makeOveralOutputEntrophy(
						entrophy.getEntrophy().getEntropyFrom(),
						entrophy.getEntrophy().getEntropyTo(),
						entrophy.getEntrophy().getWordsTwinsCount(),
						entrophy.getEntrophy().getFonemTwins().size()));
			if(aux)
				main.log(Lang.getLog("log.fileOp.save")); //"Ulozen"
			else
				main.log(Lang.getLog("log.fileOp.notSave")); //"Neulozen"
			
			main.log(Lang.getLog("log.fileOp.savingNumber") + " 2 ");//"Uladam 2. soubor"
			aux = eTwinsCreator(
					chooser.getFile().get().getExtension(), chooser.getFile().get()
									.toString(//"_jazyk1"
											Lang.getMessage("fileOp.entrophy.firstLangName")), 
					entrophy.getEntrophy().getFonemsFrom());
			if(aux)
				main.log(Lang.getLog("log.fileOp.save"));//"Ulozen"
			else
				main.log(Lang.getLog("log.fileOp.notSave"));//"Neulozen"
			
			main.log(Lang.getLog("log.fileOp.savingNumber") + " 3 "); //"Uladam 3. soubor"
			aux =  eTwinsCreator(
					chooser.getFile().get().getExtension(), chooser.getFile().get()
									.toString(//"_jazyk2"
											Lang.getMessage("fileOp.entrophy.secondLangName")),
					entrophy.getEntrophy().getFonemsTo());
			if(aux)
				main.log(Lang.getLog("log.fileOp.save")); //"Ulozen"
			else
				main.log(Lang.getLog("log.fileOp.notSave"));//"Neulozen"
			
			main.log(Lang.getLog("log.fileOp.savingNumber") + " 4 ");//"Uladam 4. soubor"
			aux = eTriplesCreator(
					chooser.getFile().get().getExtension(), chooser.getFile().get()
									.toString(//"_jazyk1-jazyk2"
											Lang.getMessage("fileOp.entrophy.fLangSLangName")),
					entrophy.getEntrophy().getFonemTwins());
			if(aux)
				main.log(Lang.getLog("log.fileOp.save"));//"Ulozen"
			else
				main.log(Lang.getLog("log.fileOp.notSave"));//"Neulozen"
			
			main.log(Lang.getLog("log.fileOp.savingNumber") + " 5 "); //"Uladam 5. soubor"
			aux = eTriplesCreator(
					chooser.getFile().get().getExtension(), chooser.getFile().get()
									.toString(//"_jazyk2-jazyk1"
											Lang.getMessage("fileOp.entrophy.sLangFLangName")),
					reverse(entrophy.getEntrophy().getFonemTwins()));
			if(aux)
				main.log(Lang.getLog("log.fileOp.save")); //"Ulozen"
			else
				main.log(Lang.getLog("log.fileOp.notSave"));//"Neulozen"
			
			return aux;
		}
		return false;
	}
	
	private boolean originCretor(final Chooser chooser, 
			final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> seq){
		switch (chooser.getFile().get().getExtension()) {
		case "csv":
			return rw.getCreator().writeGrid(
					chooser.getFile().get().toString(),
					";",
					WriteMode.WITHOUT_MODE,
					plaintext.reworkOrigin(seq)
					);
		case "txt":
			return rw.getCreator().writeGrid(
					chooser.getFile().get().toString(),
					"\t",
					WriteMode.WITHOUT_MODE,
					plaintext.reworkOrigin(seq)
					);
		case "xml":
			return rw.getCreator().writeXml(
					chooser.getFile().get().toString(),
					WriteMode.WITHOUT_MODE,
					xml.reworkOrigin(seq)
					);
		default:
			return false;
		}
	}
	
	
	private boolean calculateCreator(final Chooser chooser, 
			final IndexedSeq<Tuple3<IndexedSeq<S>, IndexedSeq<S>, Number>> seq){
		switch (chooser.getFile().get().getExtension()) {
		case "csv":
			return rw.getCreator().writeGrid(
					chooser.getFile().get().toString(),
					";",
					WriteMode.WITHOUT_MODE,
					plaintext.reworkCalculate(seq)
					);
		case "txt":
			return rw.getCreator().writeGrid(
					chooser.getFile().get().toString(),
					"\t",
					WriteMode.WITHOUT_MODE,
					plaintext.reworkCalculate(seq)
					);
		case "xml":
			return rw.getCreator().writeXml(
					chooser.getFile().get().toString(),
					WriteMode.WITHOUT_MODE,
					xml.reworkCalculate(seq)
					);
		default:
			return false;
		}
	}
	
	
	private boolean eOveralCreator(final String ext, final String name,
			final IndexedSeq<IndexedSeq<String>> seq){
		switch (ext) {
		case "csv":
			return rw.getCreator().writeGrid(
					name,
					";",
					WriteMode.WITHOUT_MODE,
					plaintext.reworkEntrophyOveral(seq)
					);
		case "txt":
			return rw.getCreator().writeGrid(
					name,
					"\t",
					WriteMode.WITHOUT_MODE,
					plaintext.reworkEntrophyOveral(seq)
					);
		case "xml":
			return rw.getCreator().writeXml(
					name,
					WriteMode.WITHOUT_MODE,
					xml.reworkEntrophyOveral(seq)
					);
		default:
			return false;
		}
	}
	
	
	private boolean eTwinsCreator(final String ext, final String name,
			final IndexedSeq<Tuple2<S,Number>> seq){
		switch (ext) {
		case "csv":
			return rw.getCreator().writeGrid(
					name,
					";",
					WriteMode.WITHOUT_MODE,
					plaintext.reworkEntrophyTwins(seq)
					);
		case "txt":
			return rw.getCreator().writeGrid(
					name,
					"\t",
					WriteMode.WITHOUT_MODE,
					plaintext.reworkEntrophyTwins(seq)
					);
		case "xml":
			return rw.getCreator().writeXml(
					name,
					WriteMode.WITHOUT_MODE,
					xml.reworkEntrophyTwins(seq)
					);
		default:
			return false;
		}
	}
	
	private boolean eTriplesCreator(final String ext, final String name, 
				final IndexedSeq<Tuple3<S, S,Number>> seq){
			switch (ext) {
			case "csv":
				return rw.getCreator().writeGrid(
						name,
						";",
						WriteMode.WITHOUT_MODE,
						plaintext.reworkEntrophyTriples(seq)
						);
			case "txt":
				return rw.getCreator().writeGrid(
						name,
						"\t",
						WriteMode.WITHOUT_MODE,
						plaintext.reworkEntrophyTriples(seq)
						);
			case "xml":
				return rw.getCreator().writeXml(
						name,
						WriteMode.WITHOUT_MODE,
						xml.reworkEntrophyTriples(seq)
						);
			default:
				return false;
			}
	}
	/*******************************************************/
	/**
	 * prohodi prvni dve polozky tuple3
	 * @param fonemTwins
	 * @return
	 */
	private IndexedSeq<Tuple3<S, S, Number>> reverse(IndexedSeq<Tuple3<S, S, Number>> fonemTwins) {
		return Stream.range(0, fonemTwins.size()).foldLeft(Array.empty(),
				(curr, i)->curr.append(new Tuple3<>(
						fonemTwins.get(i)._2,
						fonemTwins.get(i)._1,
						fonemTwins.get(i)._3)));	
	}
	
	/**
	 * vytvory vystup entropie
	 * @param firstE
	 * @param secondE
	 * @param wordsCount
	 * @param fonemCount
	 * @return
	 */
	private IndexedSeq<IndexedSeq<String>> makeOveralOutputEntrophy(
			final double firstE, final double secondE, final double wordsCount, final double fonemCount ){
		return Array.of(
					Array.of(//"Podmonina entropie prvniho jazyka: "
							Lang.getMessage("fileOp.entrophy.firstEntrophy"),
							firstE +""),
					Array.of(//"Podminena entropie druheho jazyka: "
							Lang.getMessage("fileOp.entrophy.secondEntrophy"),
							secondE +""),
					Array.of(//"Pocet dvojic slov: "
							Lang.getMessage("fileOp.entrophy.wordTwinsCount"),
							wordsCount +""),
					Array.of(//"Pocet dvojic fonem: "
							Lang.getMessage("fileOp.entrophy.phonemTwinsCount"),
							fonemCount +"")
				);
	}
	/***************************************/
	
	/**
	 * predpripravene filtry
	 * @return
	 */
	private IndexedSeq<Tuple2<String, String>> prepareFilters(){
		return Array.of(
				new Tuple2<>(//"Csv soubory (odddelene steednikem) (*.csv)"
						Lang.getMessage("filter.csv"),"csv"),
				new Tuple2<>(//"Textove soubor (oddelene tabulatorem) (*.txt)"
						Lang.getMessage("filter.txt"), "txt"),
				new Tuple2<>(//"XML soubor (*.xml)"
						Lang.getMessage("filter.xml"), "xml")
				//new Tuple2<>()
				);
	}
	
	
}
