package gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.beans.PropertyVetoException;
import java.util.Optional;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.table.AbstractTableModel;

import lang.Lang;
import structure.Structure;
import tbl.model.CalculateData;
public class Main<S> extends JFrame {
	private static final long serialVersionUID = 1L;
//	private int startW;
//	private int startH;
	private int width;
	private int height;
	private int koef;

	private Font logTextFont;
	private Font matricViewFont;
	
	final Structure<S> structure;
	/***/
	private JTextArea taLog;
	private JDesktopPane dpCentre;
	
	Action actShowInput, actCalculateInfo, actCalculateQuick, actLoad, actDetail, actEntrophy, actSave, //akce toolbaru
			actLoadGui, actLoadFile, actEdit,
			actInfoLevenshtein, actInfoHamming, actInfoJaro, actInfoJaroWinkler,
			actQuickLevenshtein, actQuickHamming, actQuickJaro, actQuickJaroWinkler,
			
			actHelp, actAbout, actSetting, actClose;
	
	
	public Main(final Structure<S> structure) {
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		//"Kalkulator Vzdalenosti Sekvenci Objektu"
		setTitle(Lang.getMessage("main.name") + " - " + structure.getName());
		setSize(800, 600);
		setIconImage(new ImageIcon(getClass().getResource("/icon.png")).getImage()) ;
	//	setIconImage(new ImageIcon("res/icon.png").getImage()) ;
	//	setIconImage(new ImageIcon("icon.png").getImage()) ;
		
		this.structure = structure;
		//vytvory logovaci area
		//"Inicializace..."
		this.taLog = new JTextArea(Lang.getLog("log.main.inicialize"));
		
		log("-- " + structure.getName() + " --");
//		this.startH = getHeight();
//		this.startW = getWidth();
		
		//nastaveni pri prekresleni menicich se promenych
		dimension();
		//vytvory centralni panel
		this.dpCentre = new JDesktopPane();
		
		//"Spoustim aplikaci..."
		log(Lang.getLog("log.main.run"));
		//vytvory akce
		createActons(structure);
		//prida horni menu
		setJMenuBar(createMenu());
		//prida obrazkove menu
		add(createToolbar(), BorderLayout.WEST);
		//prida hlavni panel
		add(dpCentre, BorderLayout.CENTER);
		//proda log
		add(createLog(), BorderLayout.SOUTH);
		
		//kontroluje zmenu okna
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent arg0) {
				dimension();
				reset();
			}
		});
		
		log(Lang.getLog("log.main.running"));
	}
	

	private void createActons(final Structure<S> structure){
		ActionsLoader<S> l = new ActionsLoader<>(this, structure);
		
		actShowInput = l.actShowInput();
		actCalculateInfo = l.actCalculateInfo();
		actInfoLevenshtein = l.actInfoLevenshtein();
		actInfoHamming = l.actInfoHamming();
		actInfoJaro = l.actInfoJaro();
		actInfoJaroWinkler = l.actInfoJaroWinkler();
		
		actCalculateQuick = l.actCalculateQuick();
		actQuickLevenshtein = l.actQuickLevenshtein();
		actQuickHamming = l.actQuickHamming();
		actQuickJaro = l.actQuickJaro();
		actQuickJaroWinkler = l.actQuickJaroWinkler();
		
		actLoad = l.actLoad();
		actSave = l.actSave();
		actLoadGui = l.actLoadGui();
		actLoadFile = l.actLoadFile();
		actEdit = l.actEdit();
		
		actDetail = l.actDetail(matricViewFont);
		actEntrophy = l.actEntrophy();
				
		actHelp = l.actHelp();
		actAbout = l.actAbout();
	//	actSetting = l.actSetting();
		actClose = l.actClose();
	}
	
	/***************** kontrola okna *********************/
	/**
	 * spocita koeficient pro velikost pisma
	 * @return
	 */
	private int calKoef(){
	//	return Math.max(Math.abs(width-startW), Math.abs(height-startH))/50;
	//	return Math.max(width-startW, height-startH)/50;
		return 15;
		
	}
	/**
	 * nastavi velikosti
	 * opakuje se pri preklesleni
	 */
	private void dimension(){
		this.width = getWidth();
		this.height = getHeight();
		this.koef = calKoef();
		
		this.logTextFont = new Font("Arial", Font.PLAIN, koef-4);
		this.matricViewFont = new Font("Arial",Font.PLAIN, koef+3);
		
		
	}
	
	/**
	 * nastavi komponenty
	 * zmeni se pri prekresleni
	 */
	private void reset(){
	//	int size = 25;
		
		//log
		taLog.setFont(this.logTextFont);
		
		dpCentre.setBorder(BorderFactory.createEtchedBorder());
		for(int i =0;i<dpCentre.getComponentCount();i++){
			getInternalAt(i).setValueSize(koef);
		}
	}
		
	/************** komponenty gui ****************/
	private JPanel createLog(){
		JPanel pLog = new JPanel();
		taLog.setRows(5);
		taLog.setEditable(false);
		pLog.setLayout(new BorderLayout()); //vyplni cely panel
		pLog.add(new JScrollPane(taLog));
		
		return pLog;
	}
	
	private JToolBar createToolbar() {
		JToolBar tb = new JToolBar(JToolBar.VERTICAL);
		tb.add(actShowInput);
		tb.add(actLoad);
		tb.add(actSave);
		tb.addSeparator();
		tb.add(actCalculateInfo);
		tb.add(actCalculateQuick);
		tb.add(actDetail);
		tb.add(actEntrophy);
		
		return tb;
	}

	private JMenuBar createMenu() {
		JMenuBar mb = new JMenuBar();	

		JMenu file = new JMenu("Soubor");
		file.add(actShowInput);
		file.addSeparator();
		file.add(new JMenuItem(actLoad));
		file.add(new JMenuItem(actLoadFile));
		file.add(new JMenuItem(actLoadGui));
		file.addSeparator();
		file.add(new JMenuItem(actEdit));
		file.addSeparator();
		file.add(new JMenuItem(actSave));
		
		JMenu quick = new JMenu(Lang.getMessage("main.menu.quick")); //"Rychlejsi vypocet"
		quick.add(new JMenuItem(actCalculateQuick));
		quick.addSeparator();
		quick.add(new JMenuItem(actQuickLevenshtein));
		quick.add(new JMenuItem(actQuickHamming));
		quick.add(new JMenuItem(actQuickJaro));
		quick.add(new JMenuItem(actQuickJaroWinkler));
		
		JMenu info = new JMenu(Lang.getMessage("main.menu.info")); //"Vypocet s detailem"
		info.add(new JMenuItem(actCalculateInfo));
		info.addSeparator();
		info.add(new JMenuItem(actInfoLevenshtein));
		info.add(new JMenuItem(actInfoHamming));
		info.add(new JMenuItem(actInfoJaro));
		info.add(new JMenuItem(actInfoJaroWinkler));
		info.addSeparator();
		info.add(new JMenuItem(actDetail));
		
		JMenu program = new JMenu(Lang.getMessage("main.menu.program")); //"Program"
		program.add(new JMenuItem(actAbout));
		program.add(new JMenuItem(actHelp));
		/*
		program.addSeparator();
		program.add(new JMenuItem(actSetting));
		*/
		program.addSeparator();
		program.add(actClose);
		
		
		mb.add(file);
		mb.add(quick);
		mb.add(info);
		mb.add(program);
		return mb;
	}
	
	/************************ dalsi metody ****************************/
	
	protected Internal getSelectedFrame(){
		return (Internal)dpCentre.getSelectedFrame();
	}
	
	@SuppressWarnings("unchecked")
	protected Optional<CalculateData<S>> getCalData(){
		Internal inter= (Internal)dpCentre.getSelectedFrame();
		if( inter.getInternalName() == InternalName.INFO )
			return  Optional.of((CalculateData<S>)inter.getTableModel().get())  ;
		return Optional.empty();
	}
	
	/**
	 * nuluje data 
	 */
	protected void clearDescTop(){
		log(Lang.getLog("log.main.cleaningDesctop"));
		dpCentre.removeAll();
		dpCentre.repaint();
		log(Lang.getLog("log.main.cleaned"));
	}
	
	
	/**
	 * zapise do logu
	 * @param message text zpravy
	 */
	protected void log(final String message){
		taLog.setText(taLog.getText() + "\n" + message);
		System.out.println("LOG MESSAGE: " + message);
	}
	/**
	 * prida dalsi okno
	 * @param name
	 * @param component
	 */
	protected void addFrame(final String name, final JComponent component, final Icon icon){
		int count = dpCentre.getAllFrames().length+1;
		Internal inter = new Internal(name, InternalName.PLAIN, component, width/3, height/2, icon);
		inter.setLocation((count-1)%10*50,(count-1)%10*50);
		dpCentre.add(inter);
		try {//nastavy nove okno jako aktivni
			inter.setSelected(true);
		} catch (PropertyVetoException e) {
			e.printStackTrace();
		}
		reset();
		
	}
	
	/**
	 * prida dalsi okno s tabulkou
	 * @param name
	 * @param tableModel
	 */
	protected void addFrame(final String name, final AbstractTableModel tableModel, 
			final InternalName interName, final Icon icon){
		int count = dpCentre.getAllFrames().length+1;
		Internal inter = new Internal(name, interName, tableModel, structure.getRenderer(),
										width/3, height/2, icon);
		inter.setLocation((count-1)%10*50,(count-1)%10*50);
		dpCentre.add(inter);
		try {//nastavy nove okno jako aktivni
			inter.setSelected(true);
		} catch (PropertyVetoException e) {
			e.printStackTrace();
		}
		reset();
		
	}
	protected  Internal getInternalAt(final int i){
		return (Internal)dpCentre.getAllFrames()[i];
	}
	
	protected  Optional<Internal> getInternalByName(final InternalName name){
		return getInternalByName(name, 0);
	}
	
	private Optional<Internal> getInternalByName(InternalName name, int i) {
		if(i==dpCentre.getComponentCount())
			return Optional.empty();
		Internal aux = (Internal)dpCentre.getAllFrames()[i];
		if(aux.getInternalName()==name)
			return Optional.of(aux);
		return getInternalByName(name, i+1);
	}


	/**
	 * zobrazi joption pane s otazkou
	 * @param query dotaz
	 * @param title titulek
	 * @param options pole moznosti
	 * @return cislo moznosti
	 */
	protected int questionDialog(String query, String title, String[] options){
		return JOptionPane.showOptionDialog(
				null,
				query,
				title,
				JOptionPane.DEFAULT_OPTION, //option type
				JOptionPane.QUESTION_MESSAGE, //message type
				null, //icon
				options, //pole moznosti
				null); //inicializacni hodnota
	}
	

}
