package gui.viewer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import lang.Lang;

public class Editor<S> {
	private List<S> values;
	private JPanel showInput;
	private int position;
	private final IndexedSeq<S> origin;
	
	private JPanel buttons;
	private Function<S, Component> fromSToComp;
	
	public Editor(final IndexedSeq<S> input/*, final JPanel buttons*/,
			final Function<S, Component> fromSToComp) {
		this.origin = input;
		this.values = fromSeqToList(input);
		this.showInput = new JPanel();
		this.position = input.size();
		this.fromSToComp = fromSToComp;
	}
	/******************************/
	
	public IndexedSeq<S> getInput(JPanel buttons){
		this.buttons = buttons;
		
		if(JOptionPane.showConfirmDialog(null, cellEditor(),
				//"Upravte retezec"
				Lang.getMessage("editor.editSequence"),
				JOptionPane.OK_CANCEL_OPTION)==JOptionPane.OK_OPTION)
			return fromListToSeq(values);
		return origin;
		
	}
	/*************************************/
	
	/**
	 * celkovy obraz
	 * @return
	 */
	private JPanel cellEditor(){  
		JPanel result = new JPanel();
		result.setLayout(new GridLayout(2,1));
		showInput.setLayout(new BorderLayout());
		showInput.add(input(), BorderLayout.CENTER);
		result.add(showInput);		
		result.add(buttons());
		return result;
	}
	
	/**
	 * ovladaci tlacitka
	 * @return
	 */
	private JPanel buttons(){
		JPanel result = new JPanel();
		result.setLayout(new GridLayout(2,1));
		result.add(buttons);
		result.add(control());
		return result;
	}
	
	/**
	 * ovladaci tlacitka
	 * @return
	 */
	private JPanel control(){
		JPanel control = new JPanel();
		//"Smazat jednu polozku"
		JButton remove = new JButton(Lang.getMessage("editor.deleteItem"));
		JButton left = new JButton("<=");
		JButton right = new JButton("=>");
		control.add(left);
		control.add(remove);
		control.add(right);
		
		remove.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				removeValueAt();
				
			}
		});
		
		left.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				movedLeft();
				
			}
		});
		
		right.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				movedRight();
				
			}
		});
		
		return control;
	}
	
	
	/**
	 * vykreneni zadani
	 * @return
	 */
	private Component input(){
		JPanel aux = new JPanel();
		aux.setLayout(new GridLayout());
		aux.setPreferredSize(new Dimension(90,100));
		JLabel pos = new JLabel("<-");
		pos.setForeground(new Color(0xff0000));
		
		for(int i =0; i<values.size();i++){
			if(i==position)
				aux.add(pos);
			aux.add(fromSToComp.apply(values.get(i)));			
		}
		if(position == values.size())
			aux.add(pos);
		
		return aux;
	}
	
	/************************************/
	
	/**
	 * posune kurzor o jedna doleva
	 */
	private void movedLeft(){
		if(position>0){
			position--;
			updateShow();
		}
	}
	
	/**
	 * posune kurzor o jedna doprava
	 */
	private void movedRight(){
		if(position<values.size()){
			position++;
			updateShow();
		}
	}
	
	/**
	 * prida hodnotu na aktualni pozici kurzoru
	 * @param value
	 */
	public Function<S, List<S>> add(){
		return (value)->{
			if(position>= values.size() || position<0){
				values.add(value);
				this.position++;
			}else{
				values.add(position, value);
				this.position++;
			}
			updateShow();
			return values;
		};
	}
	
	/**
	 * odstrani hodnotu pred kurzorem
	 */
	private void removeValueAt(){
		if(position<= values.size() && position>0){
			values.remove(position-1);
			this.position--;
			updateShow();
		}
	}
	
	/**
	 * prekresli panel se zobrazenym vstupem
	 */
	private void updateShow(){
		showInput.removeAll();
		showInput.add(input());
		showInput.updateUI();
		showInput.repaint();
	}
	
/************************************/
	private List<S> fromSeqToList(final IndexedSeq<S> input) {
		List<S> nuc = new ArrayList<>();
		for(int i =0; i<input.size();i++){
			nuc.add(input.get(i));
		}
		return nuc;
	}
	
	private IndexedSeq<S> fromListToSeq(List<S> nuc){
		return Stream.range(0, nuc.size()).foldLeft(Array.empty(),
				(curr, i)->curr.append(nuc.get(i)));
	}
	

}
