package gui.viewer;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Optional;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import io.vavr.Tuple2;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import lang.Lang;
import tbl.model.RawData;

public class GuiLoader<S>  {
	 final TableCellEditor editor;
	 final TableCellRenderer renderer;
	
	public GuiLoader( final TableCellEditor editor, final TableCellRenderer renderer) {
		this.editor = editor;
		this.renderer = renderer;
	}
	

	/**
	 * nacte nova data z rozhrani
	 * @return
	 */
	public Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> load() {
		RawData<S> raw = new RawData<>(
				Array.of(new Tuple2<>(Array.empty(),Array.empty()))
				, true);
		if(showFrame(createMessage(raw)) == JOptionPane.OK_OPTION)
			return Optional.of(raw.getTwins());
		return Optional.empty();
	}


	/**
	 * editace dat v rozhrani
	 * @param input
	 * @return
	 */
	public Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> edit(
			final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> input) {
		RawData<S> raw = new RawData<>(input, true);
		if(showFrame(createMessage(raw)) == JOptionPane.OK_OPTION)
			return Optional.of(raw.getTwins());
		return Optional.empty();
	}
	
	/**
	 * zobrazi okno pro editaci
	 * @param message
	 * @return
	 */
	private int showFrame(final JPanel message){
		return JOptionPane.showConfirmDialog(
				null,  //parent componenet - Component
				message,  //message - Object
				"",  //title - String
				JOptionPane.OK_CANCEL_OPTION); //message type - int
	}

	
	private JPanel createMessage(RawData<S> raw){
		JTable table = new JTable(raw);
		
		table.setDefaultEditor(Array.class, editor);
		table.setDefaultRenderer(Array.class, renderer);
		
		JPanel message = new JPanel();
		message.setLayout( new BorderLayout());
		JPanel buttons = new JPanel();
		JButton addLine = new JButton(//"Pridat radek"
								Lang.getMessage("guiloader.addRow"),
								new ImageIcon(getClass().getResource("/icon/icoAdd.png")));  
		JButton deleteLine = new JButton(//"Smazat radek"
								Lang.getMessage("guiloader.removeRow"),
								new ImageIcon(getClass().getResource("/icon/icoRemove.png")));  
		
		addLine.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				addLine(raw);
			}
		});
		
		deleteLine.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if(table.getSelectedRowCount() >0)
					removeLine(raw,table, table.getSelectedRows());
			}
		});
		
		buttons.add(addLine);
		buttons.add(deleteLine);
				
		message.add(buttons, BorderLayout.NORTH);
		message.add(new JScrollPane(table), BorderLayout.CENTER);
		return message;
	}
	
	/**
	 * prida radek
	 * @param raw
	 */
	private void addLine(RawData<S> raw){
		raw.addRow();
	}
	
	/**
	 * smaze vybrane radky
	 * @param raw
	 * @param rows
	 */
	private void removeLine(RawData<S> raw, JTable table, int[] rows){
		int[] modelRows = new int[rows.length];
		for(int i =0 ;i <rows.length;i++){
			modelRows[i]=  table.convertRowIndexToModel(rows[i]);
		}	
		raw.removeRows(rows);
	}
}
