package structure;

import java.awt.Component;
import java.util.function.BiFunction;
import java.util.function.Function;

import gui.viewer.TableEditor;
import gui.viewer.TableRenderer;
import io.vavr.Tuple2;
import io.vavr.collection.IndexedSeq;
import metrics.structures.CostAndInstedGI;
import rw.objects.XmlObject;

public interface Structure<S> {
	/**
	 * jmeno S
	 * @return
	 */
	public String getName();
	/**
	 * ceny operaci a pomocne objekty
	 * @return
	 */
	public CostAndInstedGI<S> getCI();
	/**
	 * odstrani pomocne objekty ze spocitanych dat
	 * kontroluje delku retezcu
	 * pro spravne fungovani entropie
	 * @return
	 */
	public Function<Tuple2<IndexedSeq<S>, IndexedSeq<S>>, Tuple2<IndexedSeq<S>, IndexedSeq<S>>> removeAuxChar();
	
	/*********************************************************************************/

	/**
	 * prevede string na sekvenci S
	 * @return
	 */
	public Function<String, IndexedSeq<S>> fromStringToSeq();
	
	/**
	 * prevede sekvenci S na string
	 * @return
	 */
	public Function<IndexedSeq<S>, String> fromSeqToString();
	
	/**
	 * prevede sekvenci S na xml objekt
	 * @return
	 */
	public BiFunction<IndexedSeq<S>, String, XmlObject> fromSeqToObject();
	
	/**
	 * prevede S na xml objekt (u plaintext - toString()
	 * @return
	 */
	public BiFunction<S, String, XmlObject> fromSToObject();
	
	/**
	 * prevede xml objekt na sekvenci S
	 * @return
	 */
	public Function<XmlObject, IndexedSeq<S>> fromObjectToSeq();
	
	/**
	 * prevede string na S
	 * @return
	 */
	public Function<String, S> fromStringToS();
	
	/**
	 * prevede S na componentu gui
	 * pro vykreslovani
	 * @return
	 */
	public Function<S, Component> fromSToComp();
	
	/*********************************************************/
	/**
	 * definice povolenych souborovych filtru
	 * @return
	 */
	public IndexedSeq<Tuple2<String, String>> getFilters();
	
	
	/**
	 * pro upravu inputu
	 * @return
	 */
	public TableEditor getEditor();
	
	/**
	 * pro spravne zobrazeni
	 * @return
	 */
	public TableRenderer getRenderer();
	
	
	
}
