package structure.character;


import java.awt.Component;
import java.util.function.BiFunction;
import java.util.function.Function;

import javax.swing.JLabel;

import gui.viewer.TableEditor;
import gui.viewer.TableRenderer;
import io.vavr.Tuple2;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import lang.Lang;
import metrics.structures.CostAndInstedGI;
import rw.objects.XmlObject;
import structure.Structure;

public class CharacterStructure implements Structure<Character> {

	
	@Override
	public String getName() {
		//"Znaky"
		return Lang.getMessage("run.selectObject.character");
	}

	@Override
	public CostAndInstedGI<Character> getCI() {
		return new CostAndInstedGI<Character>(
				(a)->'-',
				(a)->'-',
				(a)->Character.toUpperCase(a),
				(a)->Character.toUpperCase(a),
				1.0,
				1.0,
				1.0,
				1.0);
	}

	
	@Override
	public Function<Tuple2<IndexedSeq<Character>, IndexedSeq<Character>>, Tuple2<IndexedSeq<Character>, IndexedSeq<Character>>>
				removeAuxChar() {
		return (a)->new Tuple2<>(
						removeAuxChar(a._1),
						removeAuxChar(a._2)
					);
	}

	/****************************************/

	@Override
	public Function<String, IndexedSeq<Character>> fromStringToSeq() {
		return (string)->Stream.range(0, string.length()).foldLeft(Array.empty(),
				(curr, i)->{
					return curr.append( 
							fromStringToS().apply(string.charAt(i)+"") );
				});
	}


	@Override
	public Function<IndexedSeq<Character>, String> fromSeqToString() {
		return (seq)->
			Stream.range(0, seq.size()).foldLeft("",
				(curr, i)-> curr + seq.get(i).toString() );
	}

	
	@Override
	public BiFunction<IndexedSeq<Character>, String, XmlObject> fromSeqToObject() {
		return (value, name)-> new XmlObject(name, 
				fromSeqToString().apply(value));
	}


	@Override
	public BiFunction<Character, String, XmlObject> fromSToObject() {
		return (value, name)-> new XmlObject(name, value.toString() );
	}


	@Override
	public Function<XmlObject, IndexedSeq<Character>> fromObjectToSeq() {
		return (xmlObj)-> fromStringToSeq().apply(
							xmlObj.getValue().orElse(""));
	}
	
	@Override
	public Function<String, Character> fromStringToS() {
		return (string)->string.toLowerCase().charAt(0);
			
	}
	

	@Override
	public Function<Character, Component> fromSToComp() {
		return (value)->new JLabel(value.toString().toUpperCase());
	}

	
	/*************************************************/

	@Override
	public IndexedSeq<Tuple2<String, String>> getFilters() {
		return Array.of(
				new Tuple2<>(//"Csv soubory (odddelene strednekem) (*.csv)"
						Lang.getMessage("filter.csv"),"csv"),
				new Tuple2<>(//"Textove soubor (oddelene tabulatorem) (*.txt)"
						Lang.getMessage("filter.txt"), "txt"),
				new Tuple2<>(//"XML soubor (*.xml)"
						Lang.getMessage("filter.xml"), "xml")
				//new Tuple2<>()
				);
	}


	@Override
	public TableEditor getEditor() {
		return new CharacterCellEditor(
				fromStringToSeq(),
				fromSeqToString());
	}


	@Override
	public TableRenderer getRenderer() {
		return new CharacterCellRenderer(
				fromSeqToString() );
	}

	

/**************************************************************/
	
	private IndexedSeq<Character> removeAuxChar(final IndexedSeq<Character> chars){
		return Stream.range(0, chars.length()).foldLeft(Array.empty(),
				(curr, i)->{
					if(chars.get(i) == '-')
						return curr.append(' ');
					return curr.append(Character.toLowerCase(chars.get(i)));
				});
	}


	
}
