package structure.musicnotes;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.function.Function;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.border.Border;

import gui.viewer.TableRenderer;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import musicnotes.mark.Mark;

public class MusicnotesCellRenderer extends JPanel implements TableRenderer {
	private static final long serialVersionUID = 1L;
	
	private Border unselectedBorder = null;
	private Border selectedBorder = null;
	private boolean isBordered = true;
	
	private Dimension compPrefeDim= new Dimension(60,60);
	
	private final Function<Mark, Component> fromSToComp;
	
	public MusicnotesCellRenderer(final Function<Mark, Component> fromSToComp) {
		this.fromSToComp = fromSToComp;
		setOpaque(true);
		setLayout(new GridLayout());
	}
	
	@Override
	public void setSize(int koef) {
		this.repaint();
	}

	@SuppressWarnings("unchecked")
	@Override
	public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row, 
			int column) {
		
		//odstaneni predeslych panelu
		removeAll();
		
		
		if(!(value instanceof Array))
			add(new JLabel(value.toString()));
		else
			addComp((IndexedSeq<Mark>)value );
		
		setBackground(table.getBackground());
		if(isBordered){
			if(isSelected){
				if(selectedBorder == null){
					selectedBorder = BorderFactory.createMatteBorder(2, 5, 2, 5, table.getSelectionBackground());
				}
				setBorder(selectedBorder);
			}else{
				if(unselectedBorder == null){
					unselectedBorder = BorderFactory.createMatteBorder(2, 5, 2, 5, table.getBackground());
				}
				setBorder(unselectedBorder);
			}
		}
		return this;
	}
	
	private void addComp(final IndexedSeq<Mark> marks){
		for(int i = 0;i <marks.length(); i++){
			Component com =fromSToComp.apply(marks.get(i));
			com.setPreferredSize(compPrefeDim);
			this.add(com);
		}
	}

	@Override
	public int getDefaultRowHeight() {
		return 90;
	}


}
