package structure.musicnotes;

import java.awt.Component;
import java.util.Optional;
import java.util.function.BiFunction;
import java.util.function.Function;


import gui.viewer.TableEditor;
import gui.viewer.TableRenderer;
import io.vavr.Tuple2;
import io.vavr.Tuple5;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import lang.Lang;
import metrics.structures.CostAndInstedGI;
import musicnotes.mark.Mark;
import musicnotes.mark.Note;
import musicnotes.mark.OtherMarks;
import musicnotes.mark.Pause;
import musicnotes.mark.enums.Name;
import musicnotes.mark.enums.Octave;
import musicnotes.mark.enums.Shift;
import musicnotes.mark.enums.Time;
import rw.objects.XmlObject;
import musicnotes.mark.Accord;
import musicnotes.mark.EmptyMark;
import structure.Structure;
import structure.musicnotes.data.MusicnotesPanel;
import structure.musicnotes.data.NotesEnumsSwicher;

public class MusicnotesStructure implements Structure<Mark> {


	
		
	@Override
	public String getName() {
		//"Noty"
		return Lang.getMessage("run.selectObject.note");
	}

	
	@Override
	public CostAndInstedGI<Mark> getCI() {
		return new CostAndInstedGI<>(
				(a)-> new EmptyMark(),
				(a)-> new EmptyMark(),
				(a)->a,
				(a)->a,
				1.0,
				1.0,
				1.0,
				1.0);
	}
	


	@Override
	public Function<Tuple2<IndexedSeq<Mark>, IndexedSeq<Mark>>, Tuple2<IndexedSeq<Mark>, IndexedSeq<Mark>>> removeAuxChar() {
		return (a)->a;
	}


	/****************************************************/
	
	@Override
	public Function<String, IndexedSeq<Mark>> fromStringToSeq() {
		return (string)->Stream.range(0, string.length()).foldLeft(Array.empty(),
				(curr, i)->{
					if(string.charAt(i)=='{' && i<string.length()-1)
						return curr.append(
								fromStringToS().apply(getPartOf(string, i+1, '}'))
								);
					return curr;
				});
	}
	


	@Override
	public Function<String, Mark> fromStringToS() {
		return (mark)->{
			if(mark.length()>0)
				switch (mark.charAt(0)) {
				case '('://pauza
					return new Pause(
							NotesEnumsSwicher.switchTime(getPartOf(mark, 0, '}'))
							);
				case ' ': //prazdna
					return new EmptyMark();
					
				case '[': //akord
					String aux = getPartOf(mark, 0, '}');
					return new Accord(Stream.range(0, aux.length()).foldLeft(Array.empty(),
							(curr, i)->{
								if(aux.charAt(i)=='['&& i<aux.length()-1)
									return curr.append(
											makeNote(getPartOf(aux, i, ']')));
								return curr;
							}));
				default: 
					return makeOther( getPartOf(mark, 0, '}'));
				}
			return new EmptyMark();
			
		};	
	}
	


	@Override
	public Function<IndexedSeq<Mark>, String> fromSeqToString() {
		return (seq)->Stream.range(0, seq.size()).foldLeft("",
				(curr, i)-> curr + seq.get(i).toString() );
	}

	
	@Override
	public BiFunction<IndexedSeq<Mark>, String, XmlObject> fromSeqToObject() {
		return (marks, name)->new XmlObject(name)
				.withReferences(
						Stream.range(0, marks.size()).foldLeft(Array.empty(),
								(actual, i)->{
									return actual.append(makeOneMark(marks.get(i), ""));
								})
				);
	}



	@Override
	public BiFunction<Mark, String, XmlObject> fromSToObject() {
		return (mark, name)->makeOneMark(mark, name+"-"); 
	}



	@Override
	public Function<XmlObject, IndexedSeq<Mark>> fromObjectToSeq() {
		return (obj)->{
			if(obj.getReferences().equals(Optional.empty()))
				return Array.empty();
			return Stream.range(0, obj.getReferences().get().size()).foldLeft(Array.empty(),
					(curr, i)->{
						if(!obj.getReferences().get().get(i).getReferences().equals(Optional.empty()))
								return curr.append( new Accord(
									fromOneNote(obj.getReferences().get().get(i).getReferences().get())
									));
							
						if(!obj.getReferences().get().get(i).getAttributes().equals(Optional.empty()))
								return curr.append(new Pause(
										NotesEnumsSwicher.switchTime(obj.getReferences().get().get(i).getAttributes().get().get(0)._2)
										));
							
						if(!obj.getReferences().get().get(i).getValue().equals(Optional.empty()))
								return curr.append( new OtherMarks(
									NotesEnumsSwicher.switchOther(obj.getReferences().get().get(i).getValue().get())
									));
							
						return curr.append(new EmptyMark());
					});
		};
	}
	

	@Override
	public Function<Mark, Component> fromSToComp() {
	//	return (mark)->new JLabel("X");
		return (mark)->new MusicnotesPanel(mark);
	}

	/*****************************************************/

	@Override
	public IndexedSeq<Tuple2<String, String>> getFilters() {
		return Array.of(
				new Tuple2<>(//"Csv soubory (odddelene strednekem) (*.csv)"
						Lang.getMessage("filter.csv"),"csv"),
				new Tuple2<>(//"Textove soubor (oddelene tabulatorem) (*.txt)"
						Lang.getMessage("filter.txt"), "txt"),
				new Tuple2<>(//"XML soubor (*.xml)"
						Lang.getMessage("filter.xml"), "xml")
				//new Tuple2<>()
				);
	}



	@Override
	public TableEditor getEditor() {
		return new MusicnotesCellEditor(fromSToComp());
	}



	@Override
	public TableRenderer getRenderer() {
		return new MusicnotesCellRenderer(fromSToComp());
	}




/****************************************************************/
	
	/**
	 * vytvory OtherMark
	 * @param other
	 * @return
	 */
	private Mark makeOther(final String other){
		return NotesEnumsSwicher.makeOther(other);
	}
	/**
	 * vytvory cast retezce
	 * @param origin
	 * @param startPos
	 * @param stopChar
	 * @return
	 */
	private String getPartOf(final String origin, final int startPos, final char stopChar){
		return getPartOf(origin, startPos, stopChar, "");
		
		
	}
	private String getPartOf(final String origin, final int actualPos, final char stopChar, final String aux){
		if(actualPos == origin.length() || origin.charAt(actualPos)==stopChar)
			return aux;
		return getPartOf(origin, actualPos+1, stopChar, aux+ origin.charAt(actualPos) );
	}
	

	/**
	 * vytvory Note
	 * @param string
	 * @return
	 */
	private Note makeNote(final String string){
		String time;
		if(string.charAt(2)=='(')
			time = getPartOf(string, 2, ')')+")";
		else
			time = getPartOf(string, 3, ')')+")";
		
		Tuple2<Name, Octave> namOc = NotesEnumsSwicher.switchNameOctave(
				getPartOf(string, 1, '('));
		Time tim = NotesEnumsSwicher.switchTime(time);
		
		Tuple2<Boolean, Shift> bS = NotesEnumsSwicher.switchDotShift(
									string.charAt(string.length()-2)+""
									+string.charAt(string.length()-1));
		return new Note(namOc._1, namOc._2, tim, bS._2, bS._1.booleanValue());
	}

	/**
	 * ze sekvence xml objektu vytvory sekvenci Note
	 * @param obj
	 * @return
	 */
	private IndexedSeq<Note> fromOneNote(final IndexedSeq<XmlObject> obj){
		return Stream.range(0, obj.size()).foldLeft(Array.empty(),
				(curr, i)->{
					if(obj.get(i).equals(Optional.empty()) || obj.get(i).getAttributes().equals(Optional.empty())  )
						return curr;
					
					Tuple5<Name, Octave, Time, Boolean, Shift> aux = switchAtt(obj.get(i).getAttributes().get());
					
					return curr.append( 
							new Note(aux._1, aux._2, aux._3, aux._5, aux._4.booleanValue()));
				});
	}
	
	/**
	 * predela xmlobject atributy na atribury Note
	 * @param att
	 * @return
	 */
	private Tuple5<Name, Octave, Time, Boolean, Shift> switchAtt(IndexedSeq<Tuple2<String, String>> att){
		Tuple5<Name, Octave, Time, Boolean, Shift> aux  = new Tuple5<>(Name.NONE,Octave.NONE,Time.NONE, false ,Shift.NO  );
		return Stream.range(0, att.size()).foldLeft(aux,
				(curr, i )->{
					switch (att.get(i)._1) {
					case "name":
						return new Tuple5<>(
								NotesEnumsSwicher.switchName(att.get(i)._2),
								curr._2,
								curr._3,
								curr._4,
								curr._5
								);
					case "octave":
						return new Tuple5<>(
								curr._1,
								NotesEnumsSwicher.switchOctave(att.get(i)._2),
								curr._3,
								curr._4,
								curr._5
								);
					case "time":
						return new Tuple5<>(
								curr._1,
								curr._2,
								NotesEnumsSwicher.switchTime(att.get(i)._2),
								curr._4,
								curr._5
								);
					case "dot":
						return new Tuple5<>(
								curr._1,
								curr._2,
								curr._3,
								NotesEnumsSwicher.switchDot(att.get(i)._2),
								curr._5
								);
					case "shift":
						return new Tuple5<>(
								curr._1,
								curr._2,
								curr._3,
								curr._4,
								NotesEnumsSwicher.switchShift(att.get(i)._2)
								);
					default:
						return curr;
					}
				});
		
	}

	/**
	 * z mark vytvory xml object
	 * @param mark
	 * @param name
	 * @return
	 */
	private XmlObject makeOneMark(final Mark mark, final String name){
		//jednotlive noty v referencich
		if(mark instanceof Accord)
			return new XmlObject( name + "accord"  )
					.withReferences(
							makeOneNote(
									((Accord)mark)
									.getMark()
									)
					);
		//doba ve value
		if(mark instanceof Pause)
			return new XmlObject( name + "pause")
					.withAttributes(
						Array.of(new Tuple2<>(	"time",
								((Pause)mark)
								.getTime().toString()))
					);
		//jmeno znacky je ve value
		if(mark instanceof OtherMarks)
			return new XmlObject( name +"mark")
					.withValue(
							((OtherMarks)mark)
							.getMark().toString() 
					);
		//nema zadne vlastosti, vse empty
		if(mark instanceof EmptyMark)
			return new XmlObject( name + "empty");
		return new XmlObject( name + "");
	}
	
	/**
	 * ze sekvence Note vytvory sekvenci xml objektu
	 * @param acord
	 * @return
	 */
	private IndexedSeq<XmlObject> makeOneNote(final IndexedSeq<Note> acord){
		//jednotlive atributy v atributech
		return Stream.range(0, acord.size()).foldLeft(Array.empty(),
				(current, i)->{
					return current.append(
							new XmlObject("note")
							.withAttributes(
								Array.of(
									new Tuple2<>("name", acord.get(i).getName().toString()  ),
									new Tuple2<>("octave", acord.get(i).getOctave().toString() ),
									new Tuple2<>("time", acord.get(i).getTime().toString()),
									new Tuple2<>("dot", (acord.get(i).isDot() )?"true":"false" ),
									new Tuple2<>("shift", acord.get(i).getShift().toString())
									)	
								)
							);
				});
	}


}
