package structure.musicnotes.data;


import java.awt.Graphics;
import java.awt.Image;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import musicnotes.mark.Accord;
import musicnotes.mark.Mark;
import musicnotes.mark.Note;
import musicnotes.mark.OtherMarks;
import musicnotes.mark.Pause;

public class MusicnotesPanel extends JPanel{
	private static final long serialVersionUID = 1L;
//	private final int BACKW = 56;
	private final int BACKH = 60;
	private final int FROMUPTOIMG = 10;
	
	private final int EDGE = 6;
	private final int SPACE = 9;
	private final int LINE = 3;
	private final int FIRSTLINEPOSITION = FROMUPTOIMG + BACKH - EDGE - LINE/2;
	private final int TONE = (SPACE + LINE)/2;
	
	private Image backgroung;
	private Image auxLine;
	private final Mark mark;
	
	public MusicnotesPanel(final Mark mark) {
	//	this.width = getPreferredSize().width;
	//	this.height = getPreferredSize().height;
		this.mark = mark;
		this.backgroung = loadImage("lines");
		this.auxLine = loadImage("aux-line");
	}

	@Override
	public void paint(Graphics g) {
		super.paint(g);
		g.drawImage(backgroung, 0, FROMUPTOIMG, null);
		
		if(mark instanceof Accord){
			accord(g, (Accord)mark);
		}else if(mark instanceof Pause){
			pause(g, (Pause)mark);
		}else if(mark instanceof OtherMarks){
			other(g, (OtherMarks)mark);
		}else{
			
		}
	}
	
	private void accord(Graphics g, Accord acord){
		for(int i = 0; i < acord.getMark().size();i++){
			note(g, acord.getMark().get(i));
		}
	}
	
	private void note(Graphics g, Note note){
		final int NOTEH = 32;
		final int NOTEW = 21;
		final int FROMLEFT = 5;
		Image img = null;
		switch (note.getTime()) {
		case FULL:
			img = loadImage("n-1");
			break;
		case HALF:
			img = loadImage("n-2");
			break;
		case QUARTER:
			img = loadImage("n-4");
			break;
		case EIGHTH:
			img = loadImage("n-8");
			break;
		case SIXTEENTH:
			
			break;
		case THREETHYHT:
			
			break;
		default:
			break;
		}
		int height = 0;
		switch (note.getName()) {
		case C:
			height = FIRSTLINEPOSITION - NOTEH + 3 * TONE;
			g.drawImage(auxLine, FROMLEFT-3, FIRSTLINEPOSITION + LINE/2  + SPACE, null);
			break;
		case D:
			height = FIRSTLINEPOSITION - NOTEH + 2 * TONE;
			break;
		case E:
			height = FIRSTLINEPOSITION - NOTEH + TONE;
			break;
		case F:
			height = FIRSTLINEPOSITION - NOTEH;
			break;
		case G:
			height = FIRSTLINEPOSITION - NOTEH - TONE;
			break;
		case H:
			height = FIRSTLINEPOSITION - NOTEH - 2 * TONE;
			break;
		case A:
			height = FIRSTLINEPOSITION - NOTEH - 3 * TONE;
			break;
		
		default:
			break;
		}
		g.drawImage(img, FROMLEFT, height, null);
		
		
		if(note.isDot()){
			Image dot =  loadImage("dot");
			g.drawImage(dot, FROMLEFT + NOTEW + 2, height - LINE + NOTEH, null);
		}
		
		switch (note.getShift()) {
		case B:
			Image b =  loadImage("b");
			g.drawImage(b, FROMLEFT + NOTEW + NOTEW/2, height  , null);
			break;
		case HASH:
			Image hash  = loadImage("hash");
			g.drawImage(hash, FROMLEFT+ NOTEW + NOTEW/2, height , null);
			break;
		default:
			break;
		}
	}
	
	private void pause(Graphics g, Pause pause){
		final int FROMLEFT = 10;
		Image img = null;
		switch (pause.getTime()) {
		case FULL:
			img = loadImage("p-1-2");
			g.drawImage(img, FROMLEFT, FIRSTLINEPOSITION - SPACE *2 - LINE *2 - LINE/2, null);
			break;
		case HALF:
			img = loadImage("p-1-2");
			g.drawImage(img, FROMLEFT, FIRSTLINEPOSITION - SPACE *2 - LINE , null);
			break;
		case QUARTER:
			img = loadImage("p-3");
			g.drawImage(img, FROMLEFT, FIRSTLINEPOSITION - SPACE * 4, null);
			break;
		case EIGHTH:
			img = loadImage("p-4");
			g.drawImage(img, FROMLEFT, FIRSTLINEPOSITION - SPACE * 4, null);
			break;
		case SIXTEENTH:
			
			break;
		case THREETHYHT:
			
			break;
		default:
			break;
		}
		
	}
	
	private void other(Graphics g, OtherMarks other){
		switch (other.getMark()) {
		case CROWN:
			Image img = loadImage("crown");
			g.drawImage(img, 1, 1, null);
			break;

		default:
			break;
		}
	}
	
	private Image loadImage(final String name){
		return new ImageIcon(getClass().getResource("/noty/" + name + ".png")).getImage();
	}

	
}
