package structure.musicnotes.data;

import io.vavr.Tuple2;
import musicnotes.mark.EmptyMark;
import musicnotes.mark.Mark;
import musicnotes.mark.OtherMarks;
import musicnotes.mark.enums.Name;
import musicnotes.mark.enums.Octave;
import musicnotes.mark.enums.Other;
import musicnotes.mark.enums.Shift;
import musicnotes.mark.enums.Time;

public class NotesEnumsSwicher {
	
	public static Name switchName(final String name){
		switch (name.toLowerCase()) {
		case "c":
			return Name.C;
		case "d":
			return Name.D;
		case "e":
			return Name.E;
		case "f":
			return Name.F;
		case "g":
			return Name.G;
		case "h":
			return Name.H;
		case "a":
			return Name.A;
		default:
			return Name.NONE;
		}
	}
	
	public static Octave switchOctave(final String octave){
		switch (octave.toLowerCase()) {
		case "sub_contra":
			return Octave.SUB_CONTRA;
		case "contra":
			return Octave.CONTRA;
		case "big":
			return Octave.BIG;
		case "small":
			return Octave.SMALL;
		case "one_comma":
			return Octave.ONE_COMMA;
		case "two_comma":
			return Octave.TWO_COMMA;
		case "three_comma":
			return Octave.THREE_COMMA;
		case "four_comma":
			return Octave.FOUR_COMMA;
		case "five_comma":
			return Octave.FIVE_COMMA;
		default:
			return Octave.NONE;
		}
	}
	
	public static Time switchTime(final String time){
		switch (time.toLowerCase()) {
		//xml
		case "full":
			return Time.FULL;
		case "half":
			return Time.HALF;
		case "quarter":
			return Time.QUARTER;
		case "eighth":
			return Time.EIGHTH;
		case "sixteenth":
			return Time.SIXTEENTH;
		case "threethyht":
			return Time.THREETHYHT;
		//plain text	
		case "(1)":
			return Time.FULL;
		case "(1/2)":
			return Time.HALF;
		case "(1/4)":
			return Time.QUARTER;
		case "(1/8)":
			return Time.EIGHTH;
		case "(1/16)":
			return Time.SIXTEENTH;
		case "(1/32)":
			return Time.THREETHYHT;
		default:
			return Time.NONE;
		}
	}
	
	public static Shift switchShift(final String shift){
		switch (shift.toLowerCase()) {
		//xml
		case "no":
			return Shift.NO;
		case "hash":
			return Shift.HASH;
		case "b":
			return Shift.B;
		
		default:
			return Shift.NO;
		}
	}
	
	public static Other switchOther(final String mark){
		switch (mark.toLowerCase()) {
		//xml
		case "crown":
			return Other.CROWN;
		//plain text
		case "k":
			return Other.CROWN;
		default:
			return Other.NONE;
		}
	}
	
	public static boolean switchDot(final String dot){
		switch (dot.toLowerCase()) {
		//xml
		case "true":
			return true;
		/*//plain text
		case ".":
			return true;*/
		default: return false;
		}
	}
	
	public static Tuple2<Name, Octave> switchNameOctave(final String nameOctave){
		Name name = switchName(nameOctave.charAt(0)+"" );
		boolean upp = (nameOctave.charAt(0) == name.toString().charAt(0) );
		if(nameOctave.length()==1)
			return new Tuple2<Name, Octave>(name, (  (upp)?Octave.BIG:Octave.SMALL ));
		
		switch (nameOctave.charAt(1)) {
		case '1':
			return new Tuple2<Name, Octave>(name, (  (upp)?Octave.CONTRA:Octave.ONE_COMMA ));
		case '2':
			return new Tuple2<Name, Octave>(name, (  (upp)?Octave.SUB_CONTRA:Octave.TWO_COMMA ));
		case '3':
			return new Tuple2<Name, Octave>(name, (  (upp)?Octave.NONE:Octave.THREE_COMMA ));
		case '4':
			return new Tuple2<Name, Octave>(name, (  (upp)?Octave.NONE:Octave.FOUR_COMMA ));
		case '5':
			return new Tuple2<Name, Octave>(name,  (  (upp)?Octave.NONE:Octave.FIVE_COMMA ));
		
		default:
			return new Tuple2<Name, Octave>(name, Octave.NONE);
		}
	}
	
	public static Tuple2<Boolean, Shift> switchDotShift(final String shiftDot){
		//shiftdot podledni dva symboly pred ukoncujici zavorkou
		switch (shiftDot) {
		case ".$":
			return new Tuple2<>(true, Shift.B);
		case ".#":
			return new Tuple2<Boolean, Shift>(true, Shift.HASH);
		}
		switch (shiftDot.charAt(1)) {
		case '.':
			return new Tuple2<Boolean, Shift>(true, Shift.NO);
		case '#':
			return new Tuple2<>(false, Shift.HASH);
		case '$':
			return new Tuple2<>(false, Shift.B); 
		}
		return new Tuple2<>(false, Shift.NO); 
	}
	
	public static Mark makeOther(final String other){
		final Other ot= NotesEnumsSwicher.switchOther(other);
		if(ot==Other.NONE)
			return new EmptyMark();
		return new OtherMarks(ot);
	}

}
