package tbl.model;

import java.util.function.Function;

import javax.swing.table.AbstractTableModel;

import entrophy.EntrophyGI;
import io.vavr.Tuple2;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import lang.Lang;

public class EntrophyData<S> extends AbstractTableModel {
	private static final long serialVersionUID = 1L;
	private final EntrophyGI<S> ent;
	
	public EntrophyData(final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> in,
						final Function<Tuple2<IndexedSeq<S>, IndexedSeq<S>>, Tuple2<IndexedSeq<S>, IndexedSeq<S>>>
								removeAuxstrings) {
		this.ent = new EntrophyGI<>(in, removeAuxstrings);
	}
	
	public EntrophyData(final EntrophyGI<S> entrophy) {
		this.ent = entrophy;
	}
	
	@Override
	public Class<?> getColumnClass(int column){
		switch (column) {
		case 0: return Array.class;
		case 1: return Number.class;
		case 2: return Array.class;
		case 3: return Array.class;
		case 4: return Number.class;
		case 5: return Array.class;
		case 6: return Number.class;
		default: return Object.class;
		}
	}
	
	
	
	@Override
	public Object getValueAt(final int row,final int column) {
		switch(column){
		case 0: //znaky prvniho
			if(ent.getFonemsFrom().size()>row)
				return Array.of(
						ent.getFonemsFrom().get(row)._1);
			return "";
		case 1: //pocet znaku prvnho
			if(ent.getFonemsFrom().size()>row)
				return ent.getFonemsFrom().get(row)._2;
			return "";
		case 2:  //znaky prvnoho - do dvojic
			if(ent.getFonemTwins().size()>row)
				return Array.of(
						ent.getFonemTwins().get(row)._1);
			return "";
		case 3: //znaky druheho - do dvojic
			if(ent.getFonemTwins().size()>row)
				return Array.of(
						ent.getFonemTwins().get(row)._2);
			return "";
		case 4:  //pocet dvojic
			if(ent.getFonemTwins().size()>row)
				return ent.getFonemTwins().get(row)._3;
			return "";
		case 5: //znaky druheho
			if(ent.getFonemsTo().size()>row)
				return Array.of(
						ent.getFonemsTo().get(row)._1);
			return "";
		case 6: //pocet znaku
			if(ent.getFonemsTo().size()>row)
				return ent.getFonemsTo().get(row)._2;
			return "";
		}
		this.fireTableCellUpdated(row, column);
		return "NO DATA";
	}

	@Override
	public int getColumnCount() {
		return 7;
	}

	@Override
	public int getRowCount() {
		return Math.max(ent.getFonemTwins().size(),
						Math.max(ent.getFonemsFrom().size(), 
								 ent.getFonemsTo().size()));
	}

	
	@Override
	public String getColumnName(final int column) {
		switch (column){
		case 0: return Lang.getMessage("entrophy.firstRow");
		case 1: return Lang.getMessage("entrophy.secondRow");
		case 2: return Lang.getMessage("entrophy.thirdRow"); 
		case 3: return Lang.getMessage("entrophy.fourthRow");
		case 4: return Lang.getMessage("entrophy.fivethRow");
		case 5: return Lang.getMessage("entrophy.firstRow");
		case 6: return Lang.getMessage("entrophy.secondRow");
		default: return "";
		}
		
	}
	
	@Override
	public boolean isCellEditable(final int row, final int column) {
		return false;
	}
	
	public EntrophyGI<S> getEntrophy(){
		return ent;
	}
	
}
