package tbl.model;


import javax.swing.table.AbstractTableModel;


import io.vavr.Tuple2;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import lang.Lang;

public class RawData<S> extends AbstractTableModel {
	private static final long serialVersionUID = 1L;
	//nemuze byt final pro nacitani dat z gui
	private /*final*/ IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> twins;
	private final boolean editable;

	public RawData(final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> twins,
			final boolean editable) {
		this.twins = twins;
		this.editable = editable;
	}
	
	public IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> getTwins(){
		return twins;
	}
	
		
	@Override
	public int getColumnCount() {
		return 2;
	}

	@Override
	public int getRowCount() {
		return twins.size();
	}

	@Override
	public Object getValueAt(final int row,final int column) {
		switch(column){
		case 0: 
			return twins.get(row)._1;
		case 1: 
			return twins.get(row)._2;
		}
		this.fireTableCellUpdated(row, column);
		return "";
	}
	
	
	@Override
	public String getColumnName(final int column) {
		switch (column){
		case 0: return Lang.getMessage("raw.firstRow");
		case 1: return Lang.getMessage("raw.secondRow");		
		default: return "";
		}
				
	}
	
	@Override
	public Class<?> getColumnClass(int column) {
		switch (column) {
		case 0: return Array.class;
		case 1: return Array.class;
		default: return Object.class;
		}
	}
	
	@Override
	public boolean isCellEditable(final int row,final int clumn) {
		return editable;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void setValueAt(Object value, int row, int column) {
		if(editable){
			this.twins=setTwinsValueAt(twins, row, column, (IndexedSeq<S>)value );
			this.fireTableCellUpdated(row, column);
		}
	}
	
	/**
	 * nastavi hodnotu na danem radku 
	 * @param twins
	 * @param row
	 * @param column
	 * @param value
	 * @return
	 */
	private IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> setTwinsValueAt(
			final IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> twins,
			final int row, final int column, final IndexedSeq<S> value) {
		return Stream.range(0, twins.size()).foldLeft(Array.empty(),
				(current, i)->{
					if(row == i)
						return current.append(new Tuple2<>(
								(column == 0)?value: twins.get(i)._1,
								(column == 1)?value: twins.get(i)._2
								));
					return current.append(twins.get(i));
				} );
	}
	
	/**
	 * odstrani radek, neni immutable
	 * @param row
	 */
	public void removeRow(final int row){
		this.twins = Stream.range(0, twins.size()).foldLeft(Array.empty(),
				(current, i) ->{
					if(row == i)
						return current;
					return current.append(twins.get(i));
				});
		this.fireTableDataChanged();
	
	}
	/**
	 * odstarani pole radku
	 * @param rows
	 */
	public void removeRows(int[] rows){
		this.twins = Stream.range(0, twins.size()).foldLeft(Array.empty(),
				(current, i)->{
					if(partOf(i, rows, 0))
						return current;
					return current.append(twins.get(i));
				} );
		this.fireTableDataChanged();
	}
	
	/**
	 * prida radek
	 */
	public void addRow(){
		this.twins = twins.append(new Tuple2<>(Array.empty(), Array.empty()));
		this.fireTableDataChanged();
	}
	
	/**
	 * zjisti, zda hodnota je v poli
	 * @param value
	 * @param rows
	 * @param i
	 * @return
	 */
	private boolean partOf(int value, int[] rows, int i){
		if(rows.length==i)
			return false;
		if(value == rows[i])
			return true;
		return partOf(value, rows, i+1);
	}
	
	
}
