package tbl.reworker;

import java.util.Optional;
import java.util.function.Function;

import io.vavr.Tuple2;
import io.vavr.Tuple3;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;

public class PlainTextReworker<S> implements Reworker<S, IndexedSeq<IndexedSeq<String>>> {

	private final Function<String, IndexedSeq<S>> fromStringToSeq;
	
	private final Function<IndexedSeq<S>, String> fromSeqToString;
	
	
	public PlainTextReworker(
			final Function<String, IndexedSeq<S>> fromStringToSeq,
			final Function<IndexedSeq<S>, String> fromSeqToString) {
		this.fromStringToSeq = fromStringToSeq;
		this.fromSeqToString = fromSeqToString;
	}
	
	@Override
	public Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> reworkInput(
			Optional<IndexedSeq<IndexedSeq<String>>> object) {
		if(object.equals(Optional.empty()))
			return Optional.empty();
		return Optional.of(	
			Stream.range(0, object.get().size()).foldLeft(Array.empty(),
				(curr, i)->{
					if(object.get().get(i).size()>1)
						return curr.append(
								new Tuple2<>(
										fromStringToSeq.apply(object.get().get(i).get(0)),
										fromStringToSeq.apply(object.get().get(i).get(1)))
								);
					return curr;
				}));
	}

	@Override
	public IndexedSeq<IndexedSeq<String>> reworkOrigin(IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> seq) {
		return Stream.range(0, seq.size()).foldLeft(Array.empty(),
				(curr, i)->{
					return curr.append(
							Array.of(
								fromSeqToString.apply(seq.get(i)._1),
								fromSeqToString.apply(seq.get(i)._2)
							));
				});
	}

	@Override
	public IndexedSeq<IndexedSeq<String>> reworkCalculate(
			IndexedSeq<Tuple3<IndexedSeq<S>, IndexedSeq<S>, Number>> seq) {
		return Stream.range(0, seq.size()).foldLeft(Array.empty(),
				(curr, i)->{
					return curr.append(
							Array.of(
								fromSeqToString.apply(seq.get(i)._1),
								fromSeqToString.apply(seq.get(i)._2),
								seq.get(i)._3.toString()
							));
				});
	}

	@Override
	public IndexedSeq<IndexedSeq<String>> reworkEntrophyOveral(IndexedSeq<IndexedSeq<String>> seq) {
		return seq;
	}

	@Override
	public IndexedSeq<IndexedSeq<String>> reworkEntrophyTwins(IndexedSeq<Tuple2<S, Number>> seq) {
		return Stream.range(0, seq.size()).foldLeft(Array.empty(),
				(curr, i)->{
					return curr.append(
							Array.of(
								seq.get(i)._1.toString(),
								seq.get(i)._2.toString()
							));
				});
	}

	@Override
	public IndexedSeq<IndexedSeq<String>> reworkEntrophyTriples(IndexedSeq<Tuple3<S, S, Number>> seq) {
		return Stream.range(0, seq.size()).foldLeft(Array.empty(),
				(curr, i)->{
					return curr.append(
							Array.of(
								seq.get(i)._1.toString(),
								seq.get(i)._2.toString(),
								seq.get(i)._3.toString()
							));
				});
	}

}
