package tbl.reworker;

import java.util.Optional;
import java.util.function.BiFunction;
import java.util.function.Function;

import io.vavr.Tuple2;
import io.vavr.Tuple3;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import rw.objects.XmlObject;

public class XmlReworker<S> implements Reworker<S, XmlObject>{
	
	private final BiFunction<IndexedSeq<S>, String, XmlObject> fromSeqToObject;
	
	private final BiFunction<S, String, XmlObject> fromSToObject;
	
	private final Function<XmlObject, IndexedSeq<S>> fromObjectToSeq;
	
	public XmlReworker(
			final BiFunction<IndexedSeq<S>, String, XmlObject> fromSeqToObject,
			final BiFunction<S, String, XmlObject> fromSToObject,
			final Function<XmlObject, IndexedSeq<S>> fromObjectToSeq) {
		this.fromSeqToObject = fromSeqToObject;
		this.fromSToObject = fromSToObject;
		this.fromObjectToSeq = fromObjectToSeq;
	}
	

	@Override
	public Optional<IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>>> reworkInput(Optional<XmlObject> object) {
		if(object.equals(Optional.empty()) || object.get().getReferences().equals(Optional.empty())   )
			return Optional.empty();
		IndexedSeq<XmlObject> aux = object.get().getReferences().get();
		return Optional.of(
		Stream.range(0, aux.size()).foldLeft(Array.empty(),
				(curr, i)->{
					if(aux.get(i).getReferences().equals(Optional.empty())  || aux.get(i).getReferences().get().size()<2)
						return curr;
					return curr.append(
							new Tuple2<>(
									fromObjectToSeq.apply(aux.get(i).getReferences().get().get(0) ),
									fromObjectToSeq.apply(aux.get(i).getReferences().get().get(1) ))
							);
				}));
	}

	
	@Override
	public XmlObject reworkOrigin(IndexedSeq<Tuple2<IndexedSeq<S>, IndexedSeq<S>>> seq) {
		return new XmlObject("input-data")
				.withReferences(
					Stream.range(0, seq.size()).foldLeft(Array.empty(),
							(curr, i)->{
								return curr.append(
									new XmlObject("twins")
									   .withReferences(
											   Array.of(
												  fromSeqToObject.apply(seq.get(i)._1,"first-sequence"),
												  fromSeqToObject.apply(seq.get(i)._2,"second-sequence")
											  )
										)
								);
							})
				);
	}

	@Override
	public XmlObject reworkCalculate(IndexedSeq<Tuple3<IndexedSeq<S>, IndexedSeq<S>, Number>> seq) {
		return new XmlObject("calculated-data")
				.withReferences(
					Stream.range(0, seq.size()).foldLeft(Array.empty(),
							(curr, i)->{
								return curr.append(
									new XmlObject("twins-and-distance")
									   .withReferences(
											   Array.of(
											      fromSeqToObject.apply(seq.get(i)._1, "first-sequence"),
												  fromSeqToObject.apply(seq.get(i)._2, "second-sequence"),
												  new XmlObject("distance", seq.get(i)._3.toString())
											  )
										)
								);
							})
				);
	}

	@Override
	public XmlObject reworkEntrophyOveral(IndexedSeq<IndexedSeq<String>> seq) {
		return new XmlObject("entrophy-overall")
				.withReferences(
					Stream.range(0, seq.size()).foldLeft(Array.empty(),
							(curr, i)->{
								if(seq.size()<2)
									return curr;
								return curr.append(
									new XmlObject("attribute")
									   .withReferences(
											   Array.of(
												  new XmlObject("name", seq.get(i).get(0) ),
												  new XmlObject("value", seq.get(i).get(1))
											  )
										)
								);
							})
				);
	}

	@Override
	public XmlObject reworkEntrophyTwins(IndexedSeq<Tuple2<S, Number>> seq) {
		return new XmlObject("entrophy-one-language")
				.withReferences(
					Stream.range(0, seq.size()).foldLeft(Array.empty(),
							(curr, i)->{
								return curr.append(
									new XmlObject("character-and-count")
									   .withReferences(
											   Array.of(
												  fromSToObject.apply(seq.get(i)._1, "character"),
												  new XmlObject("count", seq.get(i)._2.toString())
											  )
										)
								);
							})
				);
	}

	@Override
	public XmlObject reworkEntrophyTriples(IndexedSeq<Tuple3<S, S, Number>> seq) {
		return new XmlObject("entrophy-both-language")
				.withReferences(
					Stream.range(0, seq.size()).foldLeft(Array.empty(),
							(curr, i)->{
								return curr.append(
									new XmlObject("characters-and-count")
									   .withReferences(
											   Array.of(
											     fromSToObject.apply(seq.get(i)._1, "first-character"),
												 fromSToObject.apply(seq.get(i)._2, "second-character"),
												  new XmlObject("count", seq.get(i)._3.toString())
											  )
										)
								);
							})
				);
	}

}
