package musicnotes.mark;

import musicnotes.mark.enums.Name;
import musicnotes.mark.enums.Octave;
import musicnotes.mark.enums.Shift;
import musicnotes.mark.enums.Time;

public class Note{
	private final Time time;
	private final Name name;
	private final Octave octave;
	private final boolean dot;
	private final Shift shift;
	
	
	public Note(final Name name, final Octave octave, final Time time,
			final Shift shift, final boolean dot) {
		
		this.time = time;
		this.name = name;
		this.octave = octave;
		this.shift = shift;
		this.dot = dot;
	}
	
	/**********************/
	
	@Override
	public String toString() {
		return "["
				+ switchOctave(this.name, this.octave)
				+ switchTime(this.time)
				+ ((this.dot)?".":"")
				+ switchShift(this.shift)
				+ "]";
	}
	
	@Override
	public boolean equals(Object obj) {
		if(obj instanceof Note){
			Note note = (Note)obj;
			if(time == note.getTime() &&
				name == note.getName() &&
				octave == note.getOctave() &&
				shift == note.getShift() &&
				dot == note.isDot())
				return true;
			return false;
		}
		return false;
	}
	
	/*********************************/
	public final Time getTime() {
		return time;
	}

	public final Name getName() {
		return name;
	}

	public final Octave getOctave() {
		return octave;
	}

	public final boolean isDot() {
		return dot;
	}

	public final Shift getShift() {
		return shift;
	}
	/**********************************/
	
	private String switchOctave(final Name name,final Octave octave){
		switch (octave) {
		case SUB_CONTRA:
			return name.toString().toUpperCase()+"2";
		case CONTRA:
			return name.toString().toUpperCase()+"1";
		case BIG:
			return name.toString().toUpperCase();
		case SMALL:
			return name.toString().toLowerCase();
		case ONE_COMMA:
			return name.toString().toLowerCase()+"1";
		case TWO_COMMA:
			return name.toString().toLowerCase()+"2";
		case THREE_COMMA:
			return name.toString().toLowerCase()+"3";
		case FOUR_COMMA:
			return name.toString().toLowerCase()+"4";
		case FIVE_COMMA:
			return name.toString().toLowerCase()+"5";
		default:
			return ""; 
		}
		
	}
	
	private String switchTime(final Time time){
		switch (time) {
		case FULL:
			return "(1)";
		case HALF:
			return "(1/2)";
		case QUARTER:
			return "(1/4)";
		case EIGHTH:
			return "(1/8)";
		case SIXTEENTH:
			return "(1/16)";
		case THREETHYHT:
			return "(1/32)";
		default:
			return "";
		}
	}
	
	private String switchShift(final Shift shift){
		switch (shift) {
		case NO:
			return "";
		case HASH:
			return "#";
		case B:
			return "$";
		default:
			return "";
		}
	}

}
