package structure.dna;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.function.Function;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTable;

import dnarna.base.NitroBase;
import dnarna.base.Nucleotid;
import gui.viewer.Editor;
import gui.viewer.TableEditor;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;

public class DnaCellEditor extends TableEditor {

	private static final long serialVersionUID = 1L;
	
	
	private final Function<Nucleotid, Component> fromSToComp;
	
	private IndexedSeq<Nucleotid> current;
	private JButton button;
	protected static final String EDIT = "Vlozit";	
	
	
	public DnaCellEditor(
			 final Function<Nucleotid, Component> fromSToComp) {
		this.fromSToComp = fromSToComp;
		
		this.current = Array.empty();
		this.button = new JButton();
		button.setActionCommand(EDIT);
		button.addActionListener(this);
		button.setBorderPainted(false);
		
	}
	
	@Override
	public Object getCellEditorValue() {
		if(current == null)
			return Array.empty();
		return current;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		Editor<Nucleotid> edit = new Editor<>(current, fromSToComp);
		if(EDIT.equals(e.getActionCommand())){
			current = edit.getInput(buttons(edit.add()));
			fireEditingStopped();
		}else{
			current = edit.getInput(buttons(edit.add()));
		}
		
	}

	@SuppressWarnings("unchecked")
	@Override
	public Component getTableCellEditorComponent(
			JTable table, Object value, boolean isSelected, int row, int column) {
		current = (IndexedSeq<Nucleotid>)value;
		return button;
	}
	
	
	private JPanel buttons(final Function<Nucleotid, List<Nucleotid>> add){
		JPanel adding = new JPanel();
		
		JButton a, c, g, u, t;
		a = new JButton("Adenin");
		c = new JButton("Cytozin");
		g = new JButton("Gvanin");
		u = new JButton("Uracil");
		t = new JButton("Thymin");
		
		adding.add(a);
		adding.add(c);
		adding.add(g);
		adding.add(u);
		adding.add(t);
		
		a.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				add.apply(new Nucleotid(NitroBase.ADENIN));
			}
		});
		
		c.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				add.apply(new Nucleotid(NitroBase.CYTOZIN));
			}
		});
		
		u.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				add.apply(new Nucleotid(NitroBase.URACIL));
			}
		});
		
		g.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				add.apply(new Nucleotid(NitroBase.GVANIN));
			}
		});

		t.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				add.apply(new Nucleotid(NitroBase.THYMIN));
			}
		});
		return adding;
	}
	
	

}
