package structure.dna;

import java.awt.Component;
import java.awt.Font;
import java.util.function.Function;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.border.Border;

import dnarna.base.Nucleotid;
import gui.viewer.TableRenderer;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;

public class DnaCellRenderer extends JLabel implements TableRenderer{
	private static final long serialVersionUID = 1L;
	
	Border unselectedBorder = null;
	Border selectedBorder = null;
	boolean isBordered = true;
	
	private final Function<IndexedSeq<Nucleotid>, String> fromSeqToString;
	
	public DnaCellRenderer(final Function<IndexedSeq<Nucleotid>, String> fromSeqToString) {
		this.fromSeqToString = fromSeqToString;
		setOpaque(true);
	}
	

	@SuppressWarnings("unchecked")
	@Override
	public Component getTableCellRendererComponent(
			JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {
				
		if(!(value instanceof Array))
			setText(value.toString());
		else
			setText(fromSeqToString.apply( (IndexedSeq<Nucleotid>)value ));
				
		setBackground(table.getBackground());
		if(isBordered){
			if(isSelected){
				if(selectedBorder == null){
					selectedBorder = BorderFactory.createMatteBorder(2, 5, 2, 5, table.getSelectionBackground());
				}
				setBorder(selectedBorder);
			}else{
				if(unselectedBorder == null){
					unselectedBorder = BorderFactory.createMatteBorder(2, 5, 2, 5, table.getBackground());
				}
				setBorder(unselectedBorder);
			}
		}
		return this;
	}

	@Override
	public void setSize(final int koef) {
		this.setFont(new Font("Arial", Font.PLAIN, koef));
		
	}


	@Override
	public int getDefaultRowHeight() {
		return 5;
	}

}
