package structure.dna;

import java.awt.Component;
import java.util.Optional;
import java.util.function.BiFunction;
import java.util.function.Function;

import javax.swing.JLabel;

import dnarna.base.NitroBase;
import dnarna.base.Nucleotid;
import gui.viewer.TableEditor;
import gui.viewer.TableRenderer;
import io.vavr.Tuple2;
import io.vavr.collection.Array;
import io.vavr.collection.IndexedSeq;
import io.vavr.collection.Stream;
import lang.Lang;
import metrics.structures.CostAndInstedGI;
import rw.objects.XmlObject;
import structure.Structure;

public class DnaStructure implements Structure<Nucleotid>{

	@Override
	public String getName() {
		//"DNA / RNA"
		return Lang.getMessage("run.selectObject.dna");	
	}

	@Override
	public CostAndInstedGI<Nucleotid> getCI() {
		return new CostAndInstedGI<>(
				(a)-> new Nucleotid(NitroBase.NONE),
				(a)-> new Nucleotid(NitroBase.NONE),
				(a)->a,
				(a)->a,
				1.0,
				1.0,
				1.0,
				1.0);
	}

	@Override
	public Function<Tuple2<IndexedSeq<Nucleotid>, IndexedSeq<Nucleotid>>, Tuple2<IndexedSeq<Nucleotid>, IndexedSeq<Nucleotid>>> removeAuxChar() {
		return (a)->a;
	}

	/************************************/
	
	@Override
	public Function<String, IndexedSeq<Nucleotid>> fromStringToSeq() {
		return (dna)->Stream.range(0, dna.length()).foldLeft(Array.empty(),
				(curr, i)->curr.append(fromStringToS().apply(dna.charAt(i)+""))
				);
	}

	@Override
	public Function<IndexedSeq<Nucleotid>, String> fromSeqToString() {
		return (dna)->Stream.range(0, dna.length()).foldLeft("",
				(curr, i)->curr + dna.get(i).toString());
	}

	@Override
	public BiFunction<IndexedSeq<Nucleotid>, String, XmlObject> fromSeqToObject() {
		return (dna, name)->new XmlObject(name)
				.withValue( fromSeqToString().apply(dna) );
	}

	@Override
	public BiFunction<Nucleotid, String, XmlObject> fromSToObject() {
		return (base, name)-> new XmlObject(name).withValue(base.toString());
	}

	@Override
	public Function<XmlObject, IndexedSeq<Nucleotid>> fromObjectToSeq() {
		return (obj)->{
			if(obj.getValue().equals(Optional.empty()))
				return Array.empty();
			return fromStringToSeq().apply(obj.getValue().get());
		};
	}

	@Override
	public Function<String, Nucleotid> fromStringToS() {
		return (dna)->{
			switch (dna.toLowerCase()) {
			case "a":
				return new Nucleotid(NitroBase.ADENIN);
			case "t":
				return new Nucleotid(NitroBase.THYMIN);
			case "g":
				return new Nucleotid(NitroBase.GVANIN);
			case "c":
				return new Nucleotid(NitroBase.CYTOZIN);
			case "u":
				return new Nucleotid(NitroBase.URACIL);
				
			default:
				return new Nucleotid(NitroBase.NONE);
			}
		};
	}

	@Override
	public Function<Nucleotid, Component> fromSToComp() {
		return (value)->new JLabel(value.toString().toUpperCase());
	}

	/****************************************/
	
	@Override
	public IndexedSeq<Tuple2<String, String>> getFilters() {
		return Array.of(
				new Tuple2<>(//"Csv soubory (odddelene strednekem) (*.csv)"
						Lang.getMessage("filter.csv"),"csv"),
				new Tuple2<>(//"Textove soubor (oddelene tabulatorem) (*.txt)"
						Lang.getMessage("filter.txt"), "txt"),
				new Tuple2<>(//"XML soubor (*.xml)"
						Lang.getMessage("filter.xml"), "xml")
				//new Tuple2<>()
				);
	}

	@Override
	public TableEditor getEditor() {
		return new DnaCellEditor(fromSToComp());
	}

	@Override
	public TableRenderer getRenderer() {
		return new DnaCellRenderer(fromSeqToString());
	}

}
